"""
Figure generation sample speckle patterns
"""
#%%-----------------------------------------------------------------------------
# imports
#-------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
plt.ion()

import h5py

#plotting helpers
from lib.plotting import * 

import textwrap
#%%-----------------------------------------------------------------------------
# setup samples names
#-------------------------------------------------------------------------------
##surface preparation method and roughtness:
lapping = ( 0.55, 1.6, 10 )
grinding = ( 0.55, 1.6, 10 )
h_milling = ( 2.5, 16, 50 )
v_milling = ( 2.5, 16, 50 )
turning = ( 2.5, 16, 50 )

surfaces = { 'flat lapping reaming':lapping, 
            'grinding': grinding,
            'horizontal milling': h_milling,
            'vertical milling' : v_milling,
            'turning': turning  }


#%%-----------------------------------------------------------------------------
# samples figure
#-------------------------------------------------------------------------------
data = h5py.File('..\data\surface samples.hdf5', 'r')

#methods, Rz to show
methods = ( ('horizontal milling', (2.5,16,50)),
            ('vertical milling', (2.5,16,50)),
            ('turning', (2.5,16,50)),
            ('flat lapping reaming', (0.55,1.6,10)),
            ('grinding', (0.55,1.6,10))  )
            
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

fig = plt.figure( 'Sample speckle patterns', figsize=(8,11))

ag = axis_grid( fig, rect=(0.05,0.05,0.9,0.95), rowscols=(5,3), pad=(0.02,0.02))

ax_idx = 0
for method, Rzs in methods:
    print method
    
    group = data[method]
    
    group_imgs = group['imgs']
    group_Rzs = list(group['Rzs'])
    
    for Rz in Rzs:
        print Rz
        idx = group_Rzs.index(Rz)
        img = group_imgs[idx]

        ax = ag[ax_idx]
        im = ax.imshow( img[0:256,0:256] , interpolation='nearest')
        #im.set_clim(0,img.max())
        im.set_clim(0,255)
        title = method
        title = '\n'.join(textwrap.wrap(title, 12))+'\n$R_z={0}\mu m$'.format(Rz)
        ax.text(3,10,title,va='top', color='w')
        #ax.set_title(method + '\n Rz {0}'.format(Rz))
        ax_idx+=1
        
        #disable ticks/labels
        ax.set_xticks( ())
        ax.set_yticks( ())

        print method, Rz, img.min(), img.max(), img.mean()
        
data.close()


#%%-----------------------------------------------------------------------------
# Generate small pngs for samples figure
#-------------------------------------------------------------------------------

data = h5py.File('..\data\surface samples.hdf5', 'r')

#methods, Rz to show
methods = ( ('horizontal milling', (2.5,16,50)),
            ('vertical milling', (2.5,16,50)),
            ('turning', (2.5,16,50)),
            ('flat lapping reaming', (0.55,1.6,10)),
            ('grinding', (0.55,1.6,10))  )
            
for method, Rzs in methods:
    print method
    
    group = data[method]
    group_imgs = group['imgs']
    group_Rzs = list(group['Rzs'])
    
    for Rz in Rzs:
        print Rz
        idx = group_Rzs.index(Rz)
        img = group_imgs[idx]
        
        name = '{0} Rz={1}'.format(method,Rz)

        f=plt.figure(name, figsize=(1,0.44))
        f.set
        ax = f.add_axes((0,0,1,1))
        ax.set_xticks( ())
        ax.set_yticks( ())
        im = ax.imshow( img[0:110,0:250] , interpolation='nearest')
        ax.set_aspect('equal')
        f.set_size_inches(1,0.44)
        f.savefig('..\figures\samples\pattern '+name+'.png', dpi=600)
        