"""
Generate figures for initial translation investigations
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------

import matplotlib.pyplot as plt
plt.ion()

from lib import plotting
from lib import misc
import h5py

#%%-----------------------------------------------------------------------------
# Load results for figures
#-------------------------------------------------------------------------------
data_in = h5py.File('..\\results\\4.1 methods translation.hdf5', 'r')

win_size = (512,512)

g = data_in['window size {0}'.format(win_size) ]

data_ncc   = misc.load_dict_from_hdf5( g, key='ncc' )
data_orb   = misc.load_dict_from_hdf5( g, key='orb' )
data_brisk = misc.load_dict_from_hdf5( g, key='brisk' )
data_usurf = misc.load_dict_from_hdf5( g, key='usurf' )
data_surf  = misc.load_dict_from_hdf5( g, key='surf' )

data_in.close()

#%%-----------------------------------------------------------------------------
# Paper plot - initial investigations  
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)


font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

# 1x4
f = plt.figure(figsize=(6.7,2.25))
ag = plotting.axis_grid( f, (0.05,0.3,0.9,0.6), rowscols=(1,4), pad=(0.05,0.08))
filename = '..\\figures\\initial_translation_(1x4).pdf'
titles= ('USURF', 'SURF', 'ORB', 'BRISK')

#2x2
#f = plt.figure(figsize=(3.25,3.25))
#ag = axis_grid( f, (0.12,0.15,0.84,0.86), rowscols=(2,2), pad=(0.06,0.1))
#filename = 'figures/initial_translation_(2x2).pdf'
#titles=('a)', 'b)', 'c)', 'd)')


#USURF
ax = ag[0]
ax.plot(data_usurf['Axs'], 'x', label='Feature matched $A_x$', ms=4, lw=1 )
ax.plot(data_usurf['Ays'], '.', label='Feature matched $A_y$', ms=4, lw=1 )
ax.plot(data_ncc['Axs'], '-k', label='NCC $A_x$', ms=1, lw=1 )
ax.plot(data_ncc['Ays'], '--k', label='NCC $A_y$', ms=1, lw=1 )
ax.set_title(titles[0], fontsize=8)
ax.set_xlabel('frame number')
ax.set_ylabel('Pixel shift')

#SURF
ax = ag[1]
ax.plot(data_surf['Axs'], 'x', label='SURF $A_x$', ms=4, lw=1 )
ax.plot(data_ncc['Axs'], '-k', label='NCC $A_x$', ms=1, lw=1 )
ax.plot(data_surf['Ays'], '.', label='SURF $A_y$', ms=4, lw=1 )
ax.plot(data_ncc['Ays'], '--k', label='NCC $A_y$', ms=1, lw=1 )
ax.set_title(titles[1], fontsize=8)
ax.set_xlabel('frame number')
ax.set_ylabel('Pixel shift')

#ORB
ax = ag[2]
ax.plot(data_orb['Axs'], 'x', label='ORB $A_x$', ms=4, lw=1 )
ax.plot(data_ncc['Axs'], '-k', label='NCC $A_x$', ms=1, lw=1 )
ax.plot(data_orb['Ays'], '.', label='ORB $A_y$', ms=4, lw=1 )
ax.plot(data_ncc['Ays'], '--k', label='NCC $A_y$', ms=1, lw=1 )
ax.set_title(titles[2], fontsize=8)
ax.set_xlabel('frame number')
ax.set_ylabel('Pixel shift')

#BRSIK
ax = ag[3]
ax.plot(data_brisk['Axs'], 'x', label='BRISK $A_x$', ms=4, lw=1 )
ax.plot(data_ncc['Axs'], '-k', label='NCC $A_x$', ms=1, lw=1 )
ax.plot(data_brisk['Ays'], '.', label='BRISK $A_y$', ms=4, lw=1 )
ax.plot(data_ncc['Ays'], '--k', label='NCC $A_y$', ms=1, lw=1 )
ax.set_title(titles[3], fontsize=8)
ax.set_xlabel('frame number')
ax.set_ylabel('Pixel shift')

plotting.show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)
xmin,xmax = 0, 25
ymin,ymax = -2,7
plotting.set_limits(ag, (xmin,xmax), (ymin,ymax) )

ax = ag[0]

l = ax.legend( ncol=4, frameon=False, mode="expand",
                bbox_to_anchor=(0.1, 0, 0.8, 0.12), bbox_transform=f.transFigure)

#l =  ax.legend(loc=(0.02,0.02), labelspacing=0.1, frameon=False, handletextpad=0.1, borderpad=0, ncol=2 )


f.savefig(filename, dpi=600, transparent=True)


#%%-----------------------------------------------------------------------------
# Paper plot - initial investigations errors
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)
cmap = plt.get_cmap('tab10')


# 1x4
f = plt.figure(figsize=(6.7,2.25))
ag = plotting.axis_grid( f, (0.05,0.3,0.9,0.6), rowscols=(1,4), pad=(0.05,0.08))
filename = '..\\figures\\initial_translation_error_(1x4).pdf'
titles= ('USURF', 'SURF', 'ORB', 'BRISK')

#2x2
#f = plt.figure(figsize=(3.25,3.25))
#ag = axis_grid( f, (0.12,0.15,0.84,0.86), rowscols=(2,2), pad=(0.06,0.1))
#filename = 'figures/initial_translation_error_(2x2).pdf'
#titles=('a)', 'b)', 'c)', 'd)')

Axs_ncc = data_ncc['Axs']
Ays_ncc = data_ncc['Ays']

#USURF
ax = ag[0]
errx = data_usurf['Axs']-Axs_ncc
erry = data_usurf['Ays']- Ays_ncc
ax.plot(errx, 'x', label='Feature matched error $A_x$', ms=4, lw=1)
ax.plot(erry, '.', label='Feature matched error $A_y$', ms=4, lw=1)
ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[0], fontsize=8)
ax.set_ylabel('Difference [pixels]')
ax.yaxis.labelpad = 0
ax.set_xlabel('frame number')

#SURF
ax = ag[1]
errx = data_surf['Axs']-Axs_ncc
erry = data_surf['Ays']- Ays_ncc
ax.plot(errx, 'x', label='SURF error $A_x$', ms=4, lw=1)
ax.plot(erry, '.', label='SURF error $A_y$', ms=4, lw=1)
ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[1], fontsize=8)
ax.set_xlabel('frame number')

#ORB
ax = ag[2]
errx = data_orb['Axs']-Axs_ncc
erry = data_orb['Ays']- Ays_ncc
ax.plot(errx, 'x', label='ORB error $A_x$', ms=4, lw=1 )
ax.plot(erry, '.', label='ORB error $A_y$', ms=4, lw=1)
ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[2], fontsize=8)
ax.set_xlabel('frame number')
ax.set_ylabel('Difference [pixels]')


#BRSIK
ax = ag[3]
errx = data_brisk['Axs']-Axs_ncc
erry = data_brisk['Ays']- Ays_ncc
ax.plot(errx, 'x', label='BRISK error $A_x$', ms=4, lw=1)
ax.plot(erry, '.', label='BRISK error $A_y$', ms=4, lw=1)
ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[3], fontsize=8)

xmin,xmax = 0,100
ymin,ymax = -0.3,0.3
plotting.set_limits( ag, xlimits=(xmin,xmax), ylimits=(ymin,ymax))
plotting.show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)

ax = ag[0]
l = ax.legend( ncol=2, frameon=False, mode="expand",
                bbox_to_anchor=(0.2, 0, 0.6, 0.12), bbox_transform=f.transFigure)

f.savefig(filename, dpi=600, transparent=True)

#%%-----------------------------------------------------------------------------
# pixel locking figure
#-------------------------------------------------------------------------------
data_in = h5py.File('results\\methods translation simulated.hdf5', 'r')
win_size = (256,256)
g = data_in['window size {0}'.format(win_size) ]

Axs = g['Axs'][:]
Ays = g['Ays'][:]

data_ncc   = misc.load_dict_from_hdf5( g, key='ncc' )
data_orb   = misc.load_dict_from_hdf5( g, key='orb' )
data_brisk = misc.load_dict_from_hdf5( g, key='brisk' )
data_usurf = misc.load_dict_from_hdf5( g, key='usurf' )
data_surf  = misc.load_dict_from_hdf5( g, key='surf' )

data_in.close()


font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

f = plt.figure('Pixel locking {0}'.format(win_size), figsize=(3.25,3))
ax = f.add_axes( (0.2,0.3,0.7,0.6))
ax.set_ylim(-0.2,0.2)
ax.set_xlim(-1.5,1.5)
ax.set_ylabel('Bias error [px]')
ax.yaxis.labelpad = 0
ax.set_xlabel('Simulated translation [px]')
ax.plot( Axs, data_usurf['Axs']-Axs, '^--', label='USURF', lw=1, zorder=1)
ax.plot( Axs, data_surf['Axs']-Axs, '*--', label='SURF', lw=1, zorder=2)
ax.plot( Axs, data_orb['Axs']-Axs, '.--', label='ORB', lw=1, zorder=3)
ax.plot( Axs, data_brisk['Axs']-Axs, 'x--', label='BRISK', lw=1, zorder=4)
ax.plot( Axs, data_ncc['Axs']-Axs, '-k', label='NCC', lw=1, zorder=5)

plt.legend(bbox_to_anchor=(0.77,0.7), bbox_transform=f.transFigure, frameon=False )

l = ax.legend( ncol=3, frameon=False, mode="expand",
                bbox_to_anchor=(0.1, 0.01, 0.8, 0.13), bbox_transform=f.transFigure)

f.savefig('figures//peak_locking.pdf', dpi=600)


#%%-----------------------------------------------------------------------------
# Paper plot - initial investigations errors
#-------------------------------------------------------------------------------
data_in = h5py.File('..\\results\\4.1 methods translation simulated.hdf5', 'r')
win_size = (256,256)
g = data_in['window size {0}'.format(win_size) ]

Axs = g['Axs'][:]
Ays = g['Ays'][:]

data_ncc   = misc.load_dict_from_hdf5( g, key='ncc' )
data_orb   = misc.load_dict_from_hdf5( g, key='orb' )
data_brisk = misc.load_dict_from_hdf5( g, key='brisk' )
data_usurf = misc.load_dict_from_hdf5( g, key='usurf' )
data_surf  = misc.load_dict_from_hdf5( g, key='surf' )

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)
cmap = plt.get_cmap('tab10')

# 1x4
f = plt.figure(figsize=(6.7,2.25))
ag = plotting.axis_grid( f, (0.05,0.3,0.9,0.6), rowscols=(1,4), pad=(0.05,0.08))
filename = 'figures\\peak_locking_(1x4).pdf'
titles= ('USURF', 'SURF', 'ORB', 'BRISK')

#2x2
#f = plt.figure(figsize=(3.25,3.25))
#ag = axis_grid( f, (0.12,0.15,0.84,0.86), rowscols=(2,2), pad=(0.06,0.1))
#filename = 'figures/initial_translation_error_(2x2).pdf'
#titles=('a)', 'b)', 'c)', 'd)')

#USURF
ax = ag[0]
errx = data_usurf['Axs']-Axs
erry = data_usurf['Ays']- Ays
ax.plot(Axs, errx, 'x--')
ax.plot( Axs, data_ncc['Axs']-Axs, '-k', label='NCC', lw=1, zorder=5)
#ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[0], fontsize=8)
ax.set_ylabel('Bias error [px]')
ax.yaxis.labelpad = 0
ax.set_xlabel('Translation [px]')

#SURF
ax = ag[1]
errx = data_surf['Axs']-Axs
erry = data_surf['Ays']- Ays
ax.plot(Axs, errx, 'x--')
ax.plot(Axs, data_ncc['Axs']-Axs, '-k', label='NCC', lw=1, zorder=5)
#ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[1], fontsize=8)
ax.set_ylabel('Bias error [px]')
ax.yaxis.labelpad = 0
ax.set_xlabel('Translation [px]')

#ORB
ax = ag[2]
errx = data_orb['Axs']-Axs
erry = data_orb['Ays']- Ays
ax.plot(Axs, errx, 'x--')
ax.plot(Axs, data_ncc['Axs']-Axs, '-k', label='NCC', lw=1, zorder=5)
#ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[2], fontsize=8)
ax.set_ylabel('Bias error [px]')
ax.yaxis.labelpad = 0
ax.set_xlabel('Translation [px]')


#BRSIK
ax = ag[3]
errx = data_brisk['Axs']-Axs
erry = data_brisk['Ays']- Ays
ax.plot(Axs, errx, 'x--')
ax.plot(Axs, data_ncc['Axs']-Axs, '-k', label='NCC', lw=1, zorder=5)
#ax.hlines( y=0, xmin=0, xmax=100, color='k', linestyle='dashed')
ax.set_title(titles[3], fontsize=8)
ax.set_ylabel('Bias error [px]')
ax.yaxis.labelpad = 0
ax.set_xlabel('Translation [px]')

ymin,ymax = -0.2,0.2
xmin,xmax = -1.5,1.5
plotting.set_limits( ag, xlimits=(xmin,xmax), ylimits=(ymin,ymax))
plotting.show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)

ax = ag[0]
l = ax.legend( ncol=2, frameon=False, mode="expand",
                bbox_to_anchor=(0.2, 0, 0.6, 0.12), bbox_transform=f.transFigure)

f.savefig(filename, dpi=600, transparent=True)
data_in.close()
