"""
Initial investigation of methods - experimental continuous translation
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------
import numpy as np
import cv2 as cv
import h5py

from lib import methods
from lib import misc


data = h5py.File('..\\data\\dataset1 continuous translation.hdf5', 'r')
imgs = data['imgs']
win = (0,0,512,512)     #window size to use (x0,y0,w,h)
#win = (128,128,256,256)     #window size to use (x0,y0,w,h)
#win = (192,192,128,128)     #window size to use (x0,y0,w,h)

ts = np.arange(0, imgs.shape[0])*(1/500.0) # time indices: 500fps 
N = None #max matches to use (None no limit)

#%%-----------------------------------------------------------------------------
# Continuous translation (frame to frame)
#-------------------------------------------------------------------------------
ncc = methods.NCC()
orb = methods.ORB(distance_threshold=30 )           #30
brisk = methods.BRISK(distance_threshold=60 )       #60
usurf = methods.USURF(distance_threshold=0.5, norm=cv.NORM_L1 ) #0.5
surf = methods.SURF(distance_threshold=0.5, norm=cv.NORM_L1 )   #0.5

print 'Doing NCC ----------------------------------------------------------'
img0 = imgs[0,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
for n in xrange(1, imgs.shape[0],1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
    Ax,Ay = ncc.calc_shift(img0,img1)
    #print '\tncc: {0:0.2f}, {1:0.2f}'.format(Ax,Ay)
    
    #set new reference
    img0 = img1
    
#get results as dictionary 
data_ncc = ncc.get_data() 
print 'Average time ', np.mean(data_ncc['dts']) * 1000, ' ms'

print 'Doing ORB ----------------------------------------------------------'
img0 = imgs[0,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
for n in xrange(1, imgs.shape[0],1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = orb.calc_shift_and_rotation(img0, img1, maxMatches=N)
    #print '\torb: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, orb.N0s[-1])
    
    #set new reference
    img0 = img1

data_orb = orb.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_orb['dts']) * 1000, ' ms'

print 'Doing BRISK ----------------------------------------------------------'
img0 = imgs[0,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
for n in xrange(1, imgs.shape[0],1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = brisk.calc_shift_and_rotation(img0, img1, maxMatches=N)
    #print '\tbrisk: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, brisk.N0s[-1])
    
    #set new reference
    img0 = img1
    
data_brisk = brisk.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_brisk['dts']) * 1000, ' ms'

print 'Doing USURF ----------------------------------------------------------'
img0 = imgs[0,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
for n in xrange(1, imgs.shape[0],1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = usurf.calc_shift_and_rotation(img0, img1, maxMatches=N)
    #print '\tusurf: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, usurf.N0s[-1])
    
    #set new reference
    img0 = img1

data_usurf = usurf.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_usurf['dts']) * 1000, ' ms'

print 'Doing SURF ----------------------------------------------------------'
img0 = imgs[0,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
for n in xrange(1, imgs.shape[0],1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = surf.calc_shift_and_rotation(img0, img1, maxMatches=N)
    #print '\tsurf: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, surf.N0s[-1])

    #set new reference
    img0 = img1

data_surf = surf.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_surf['dts']) * 1000, ' ms'

#%%-----------------------------------------------------------------------------
#save results to hdf5 file
#-------------------------------------------------------------------------------
out = h5py.File('..\\results\\4.1 methods translation.hdf5', 'a')
if 'window size {0}'.format(win[2:]) not in out:
    g = out.create_group('window size {0}'.format(win[2:]) )
else:
    g = out['window size {0}'.format(win[2:])]

misc.save_dict_to_hdf5( data_ncc, g, key='ncc' )
misc.save_dict_to_hdf5( data_orb, g, key='orb' )
misc.save_dict_to_hdf5( data_brisk, g, key='brisk' )
misc.save_dict_to_hdf5( data_usurf, g, key='usurf' )
misc.save_dict_to_hdf5( data_surf, g, key='surf' )

out.close()

