"""
Initial investigation of methods - artifically translated images
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------
import numpy as np
import cv2 as cv
import h5py

from lib import methods
from lib import misc

data = h5py.File('..\\data\\dataset2 random positions.hdf5', 'r')
imgs = data['imgs']

##Setup range of shifts to simulate
xshifts = np.arange(1.5,-1.55,-0.1)
yshifts = np.zeros( xshifts.shape)  
shifts = zip(xshifts,yshifts)

#Other settings
#win = (128,128,256,256)     #window size to use (x0,y0,w,h)
win = (192,192,128,128)     #window size to use (x0,y0,w,h)

N = None                    #max matches to use (None no limit)
M = 10                      #image magnification 10 for 0.1 pixel shift


#%%-----------------------------------------------------------------------------
# Loop over shifts and process all images in random positions set
#-------------------------------------------------------------------------------
ncc = methods.NCC()
orb = methods.ORB(distance_threshold=35 )
brisk = methods.BRISK(distance_threshold=60 )
usurf = methods.USURF(distance_threshold=0.5, norm=cv.NORM_L1 )
usurf.usurf.setHessianThreshold(30)

surf = methods.SURF(distance_threshold=0.5, norm=cv.NORM_L1 )
surf.surf.setHessianThreshold(30)

#store mean of Ax,Ay,theta for each method at each modelled shift
data_ncc = {'Axs':np.zeros(xshifts.shape[0]) ,
            'Ays':np.zeros(xshifts.shape[0]) ,
            'thetas':np.zeros(xshifts.shape[0]) }
data_orb = {'Axs':np.zeros(xshifts.shape[0]) ,
            'Ays':np.zeros(xshifts.shape[0]) ,
            'thetas':np.zeros(xshifts.shape[0]) }
data_brisk = {'Axs':np.zeros(xshifts.shape[0]) ,
            'Ays':np.zeros(xshifts.shape[0]) ,
            'thetas':np.zeros(xshifts.shape[0]) }
data_usurf = {'Axs':np.zeros(xshifts.shape[0]) ,
            'Ays':np.zeros(xshifts.shape[0]) ,
            'thetas':np.zeros(xshifts.shape[0]) }            
data_surf = {'Axs':np.zeros(xshifts.shape[0]) ,
            'Ays':np.zeros(xshifts.shape[0]) ,
            'thetas':np.zeros(xshifts.shape[0]) }
            
for n in range(0, len(shifts)):
    shift = shifts[n]
    print 'doing shift ---------------------------------------------', shift
    
    for i in range(0,100):
        
        #generate a shifted windows from image
        img0,img1 = misc.model_shifted_windows(imgs[i], win0=win, delta=shift, M=10)
        
        ##Do NCC processsing
        Ax,Ay = ncc.calc_shift(img0,img1)
        #print '\tncc: {0:0.2f} , {1:0.2f}'.format(Ax,Ay)
        
        ##Do ORB
        Ax,Ay,theta = orb.calc_shift_and_rotation(img0, img1, maxMatches=N)
        #print '\torb: {0:0.2f} , {1:0.2f} , {2:0.2f}'.format(Ax,Ay,theta)

        ##Do BRISK
        Ax,Ay,theta = brisk.calc_shift_and_rotation(img0, img1, maxMatches=N)
        #print '\tbrisk: {0:0.2f} , {1:0.2f} , {2:0.2f}'.format(Ax,Ay,theta)
        
        ##Do USURF
        Ax,Ay,theta = usurf.calc_shift_and_rotation(img0, img1, maxMatches=N)
        #print '\tusurf: {0:0.2f} , {1:0.2f} , {2:0.2f}'.format(Ax,Ay,theta)
        
        ##Do SURF
        Ax,Ay,theta = surf.calc_shift_and_rotation(img0, img1, maxMatches=N)
        #print '\tsurf: {0:0.2f} , {1:0.2f} , {2:0.2f}'.format(Ax,Ay,theta)
    
    #store means
    data_ncc['Axs'][n] = np.nanmean(ncc.Axs)
    data_ncc['Ays'][n] = np.nanmean(ncc.Ays)
    data_ncc['thetas'][n] = np.nanmean(ncc.thetas)
    ncc.reset()
    
    data_orb['Axs'][n] = np.nanmean(orb.Axs)
    data_orb['Ays'][n] = np.nanmean(orb.Ays)
    data_orb['thetas'][n] = np.nanmean(orb.thetas)
    orb.reset()
    
    data_brisk['Axs'][n] = np.nanmean(brisk.Axs)
    data_brisk['Ays'][n] = np.nanmean(brisk.Ays)
    data_brisk['thetas'][n] = np.nanmean(brisk.thetas)
    brisk.reset()
    
    data_usurf['Axs'][n] = np.nanmean(usurf.Axs)
    data_usurf['Ays'][n] = np.nanmean(usurf.Ays)
    data_usurf['thetas'][n] = np.nanmean(usurf.thetas)
    usurf.reset()
    
    data_surf['Axs'][n] = np.nanmean(surf.Axs)
    data_surf['Ays'][n] = np.nanmean(surf.Ays)
    data_surf['thetas'][n] = np.nanmean(surf.thetas)
    surf.reset()
    
#%%-----------------------------------------------------------------------------
#store results to output file
#-------------------------------------------------------------------------------
out = h5py.File('..\\results\\4.1 methods translation simulated.hdf5', 'a')
if 'window size {0}'.format(win[2:]) in out:
    g = out['window size {0}'.format(win[2:])]
    g.clear()
else:
    g = out.create_group('window size {0}'.format(win[2:]) )

#store applied shifts and window size etc
g.create_dataset('Axs', data=xshifts, compression='gzip')
g.create_dataset('Ays', data=yshifts, compression='gzip')
g.attrs['window size'] = win[2:]
g.attrs['magnification'] = M

misc.save_dict_to_hdf5( data_ncc , g, 'ncc')
misc.save_dict_to_hdf5( data_orb , g, 'orb')
misc.save_dict_to_hdf5( data_brisk , g, 'brisk')
misc.save_dict_to_hdf5( data_usurf , g, 'usurf')
misc.save_dict_to_hdf5( data_surf , g, 'surf')

out.close()
