"""
Generate figures for initial rotations investigations
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------

import matplotlib.pyplot as plt
plt.ion()

from lib import plotting
from lib import misc
import h5py

#%%-----------------------------------------------------------------------------
# Load results
#-------------------------------------------------------------------------------
data_in = h5py.File('..\\results\\4.2 methods rotations.hdf5', 'r')

win_size = (512,512)

g = data_in['window size {0}'.format(win_size) ]

Dangles = g['Dangles'][:]
data_orb   = misc.load_dict_from_hdf5( g, key='orb' )
data_brisk = misc.load_dict_from_hdf5( g, key='brisk' )
data_usurf = misc.load_dict_from_hdf5( g, key='usurf' )
data_surf  = misc.load_dict_from_hdf5( g, key='surf' )

data_in.close()
#%%-----------------------------------------------------------------------------
# Paper plot - initial investigations  
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)


f = plt.figure('Rotations {0}'.format(win_size), figsize=(3.25,4))

ang0, ang1 = -35, 35
err = 0.3
ms = 3
ag = plotting.axis_grid( f, (0.15,0.1,0.8,0.9), rowscols=(3,1), pad=(0.05,0.05))
plotting.show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)


ax1 = ag[0]
ax2 = ag[1]
ax3 = ag[2]
ax1.plot( Dangles, data_usurf['thetas'], '^', label='USURF', ms=ms, zorder=4)
ax1.plot( Dangles, data_surf['thetas'], '*', label='SURF', ms=ms, zorder=3)
ax1.plot( Dangles, data_orb['thetas'], '.', label='ORB', ms=ms, zorder=1)
ax1.plot( Dangles, data_brisk['thetas'], 'x', label='BRISK', ms=ms, zorder=2)
ax1.set_xlim(ang0,ang1)
ax1.set_ylim(ang0,ang1)
ax1.set_ylabel('Measured rotation\n[degrees]')
ax1.yaxis.labelpad = 5
plt.legend(ncol=2, loc=(0.02,0.55), numpoints = 1, scatterpoints=1, 
            labelspacing=0.00,columnspacing=1.20, frameon=False)

#error
ax2.plot( Dangles, data_usurf['thetas']-Dangles, '^', label='USURF', ms=ms, zorder=4)
ax2.plot( Dangles, data_surf['thetas']-Dangles, '*', label='SURF', ms=ms, zorder=3)
ax2.plot( Dangles, data_orb['thetas']-Dangles, '.', label='ORB', ms=ms, zorder=1)
ax2.plot( Dangles, data_brisk['thetas']-Dangles, 'x', label='BRISK', ms=ms, zorder=2)
ax2.set_ylim(-err,err)
ax2.set_xlim(ang0,ang1)
ax2.set_ylabel('Angle error\n[degrees]')
ax2.yaxis.labelpad = 2
#ax2.set_xlabel('Rotation angle [degrees]')
ax2.hlines(0,-180,180, 'k', '--')

#Number of features
ax3.plot( Dangles, data_usurf['Nfms'], '^', label='USURF', ms=ms, zorder=3)
ax3.plot( Dangles, data_surf['Nfms'], '*', label='SURF', ms=ms, zorder=4)
ax3.plot( Dangles, data_orb['Nfms'], '.', label='ORB', ms=ms, zorder=1)
ax3.plot( Dangles, data_brisk['Nfms'], 'x', label='BRISK', ms=ms, zorder=2)
ax3.set_yscale('log')
ax3.set_ylim(0,10000)
ax3.set_xlim(ang0,ang1)
ax3.set_ylabel('N matches')
ax3.set_xlabel('Rotation angle [degrees]')
ax3.yaxis.labelpad = 5


l =  ax1.legend(loc=(0.02,0.5), labelspacing=0.1, frameon=False, 
                handletextpad=0.1, borderpad=0, ncol=1 )

f.savefig('..\\figures\\initial_rotation.pdf', dpi=600, transparent=True)

#%%-----------------------------------------------------------------------------
# Paper plot - initial investigations  
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)


f = plt.figure(figsize=(6.7,3))

ang0, ang1 = -32, 32
ms = 3
ag = plotting.axis_grid( f, (0.0,0.15,0.9,0.95), rowscols=(2,2), pad=(0.15,0.18))

ax = ag[0]
ax.plot( Dangles, data_usurf['thetas'], '^', label='USURF', ms=ms, zorder=4)
ax.plot( Dangles, data_surf['thetas'], '*', label='SURF', ms=ms, zorder=3)
ax.plot( Dangles, data_orb['thetas'], '.', label='ORB', ms=ms, zorder=1)
ax.plot( Dangles, data_brisk['thetas'], 'x', label='BRISK', ms=ms, zorder=2)
ax.set_xlim(ang0,ang1)
ax.set_ylim(ang0,ang1)
ax.set_xlabel('Rotation angle [degrees]')
ax.set_ylabel('Measured rotation\n[degrees]')
ax.yaxis.labelpad = 5
ax.set_title('a)', fontsize=8)

#Nmber of features
ax = ag[1]
ax.plot( Dangles, data_usurf['Nfms'], '^', label='USURF', ms=ms, zorder=3)
ax.plot( Dangles, data_surf['Nfms'], '*', label='SURF', ms=ms, zorder=4)
ax.plot( Dangles, data_orb['Nfms'], '.', label='ORB', ms=ms, zorder=1)
ax.plot( Dangles, data_brisk['Nfms'], 'x', label='BRISK', ms=ms, zorder=2)
ax.set_yscale('log')
ax.set_ylim(0,10000)
ax.set_xlim(ang0,ang1)
ax.set_ylabel('N matches')
ax.set_xlabel('Rotation angle [degrees]')
ax.yaxis.labelpad = 5
ax.set_title('b)', fontsize=8)

#error
err_usurf = data_usurf['thetas']-Dangles
err_surf  = data_surf['thetas']-Dangles
err_orb   = data_orb['thetas']-Dangles
err_brisk = data_brisk['thetas']-Dangles

ax = ag[2]
ax.plot( Dangles, err_usurf, '^', label='USURF', ms=ms, zorder=4)
ax.plot( Dangles, err_surf, '*', label='SURF', ms=ms, zorder=3)
ax.plot( Dangles, err_orb, '.', label='ORB', ms=ms, zorder=1)
ax.plot( Dangles, err_brisk, 'x', label='BRISK', ms=ms, zorder=2)
ax.set_ylim(-0.3,0.3)
ax.set_xlim(ang0,ang1)
ax.set_ylabel('Angle error\n[degrees]')
ax.yaxis.labelpad = 2
ax.set_xlabel('Rotation angle [degrees]')
ax.hlines(0,-180,180, 'k', '--')
ax.set_title('c)', fontsize=8)

#error 2
ax = ag[3]
ax.plot( Dangles, err_usurf, '^', label='USURF', ms=ms, zorder=4)
ax.plot( Dangles, err_surf, '*', label='SURF', ms=ms, zorder=3)
ax.plot( Dangles, err_orb, '.', label='ORB', ms=ms, zorder=1)
ax.plot( Dangles, err_brisk, 'x', label='BRISK', ms=ms, zorder=2)
ax.set_ylim(-0.03,0.03)
ax.set_xlim(ang0,ang1)
ax.set_ylabel('Angle error\n[degrees]')
ax.yaxis.labelpad = 2
ax.set_xlabel('Rotation angle [degrees]')
ax.hlines(0,-180,180, 'k', '--')
ax.set_title('d)', fontsize=8)

plotting.show_outside_xaxis_only(ag, labels = False, ticklabels=False, ticks=True)
plotting.set_xticks( ag, (-30,-15,0,15,30))
ax = ag[0]
l =  ax.legend(loc=(0.02,0.45), labelspacing=0.1, frameon=False, 
                handletextpad=0.1, borderpad=0, ncol=1 )

f.savefig('..\\figures\\initial_rotation.pdf', dpi=600, transparent=True)



#%%-----------------------------------------------------------------------------
# slides plot - 1 method only 
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)


f = plt.figure(figsize=(3.6,3.2))

ang0, ang1 = -15, 15
ms = 3
ag = plotting.axis_grid( f, (0.05,0.15,0.85,0.85), rowscols=(2,1), pad=(0.15,0.1))

ax = ag[0]
#ax.plot( Dangles, data_usurf['thetas'], '^', label='USURF', ms=ms, zorder=4)
#ax.plot( Dangles, data_surf['thetas'], '*', label='SURF', ms=ms, zorder=3)
ax.plot( Dangles, data_orb['thetas'], '.', label='ORB', ms=ms, zorder=1)
#ax.plot( Dangles, data_brisk['thetas'], 'x', label='BRISK', ms=ms, zorder=2)
ax.set_xlim(ang0,ang1)
ax.set_ylim(ang0,ang1)
ax.set_xlabel('Rotation angle [degrees]')
ax.set_ylabel('Measured rotation\n[degrees]')
ax.yaxis.labelpad = 5
#
#error
err_usurf = data_usurf['thetas']-Dangles
err_surf  = data_surf['thetas']-Dangles
err_orb   = data_orb['thetas']-Dangles
err_brisk = data_brisk['thetas']-Dangles

#error 2
ax = ag[1]
#ax.plot( Dangles, err_usurf, '^', label='USURF', ms=ms, zorder=4)
#ax.plot( Dangles, err_surf, '*', label='SURF', ms=ms, zorder=3)
ax.plot( Dangles, err_orb, '.', label='ORB', ms=ms, zorder=1)
#ax.plot( Dangles, err_brisk, 'x', label='BRISK', ms=ms, zorder=2)
ax.set_ylim(-0.03,0.03)
ax.set_xlim(ang0,ang1)
ax.set_ylabel('Angle error\n[degrees]')
ax.yaxis.labelpad = 2
ax.set_xlabel('Rotation angle [degrees]')
ax.hlines(0,-180,180, 'k', '--')
#ax.set_title('b) Measurement error ', fontsize=8)

plotting.show_outside_xaxis_only(ag, labels = False, ticklabels=False, ticks=True)
#plotting.set_xticks( ag, (-30,-15,0,15,30))
#ax = ag[0]
#l =  ax.legend(loc=(0.02,0.45), labelspacing=0.1, frameon=False, 
#                handletextpad=0.1, borderpad=0, ncol=1 )

#f.savefig('figures\\slides_rotation.png', dpi=600, transparent=True)
