"""
Initial investigation of methods - experimental rotations
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------
import numpy as np
import cv2 as cv
import h5py

from lib import methods
from lib import misc


data = h5py.File('..\\data\\dataset3 stepped rotations.hdf5', 'r')
imgs = data['imgs']
angles = data['angles'][:]  #[:] copy to array.

ref_i   = 360   #refernce to this frame


win = (0,0,512,512)     #window size to use (x0,y0,w,h)
#win = (128,128,256,256)     #window size to use (x0,y0,w,h)
#win = (192,192,128,128)     #window size to use (x0,y0,w,h)

ts = np.arange(0, imgs.shape[0])*(1/500.0) # time indices: 500fps 
N = None #max matches to use (None no limit)

#%%-----------------------------------------------------------------------------
# Rotations compared to fixed reference
#-------------------------------------------------------------------------------
ncc = methods.NCC()
orb = methods.ORB(distance_threshold=35 )
brisk = methods.BRISK(distance_threshold=60 )
usurf = methods.USURF(distance_threshold=0.5, norm=cv.NORM_L1 )
surf = methods.SURF(distance_threshold=0.5, norm=cv.NORM_L1 )

#fixed reference image
img0 = imgs[ref_i,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]
Dangles = angles - angles[ref_i]

d0 = 0
d1 = wins.shape[0]

print 'Doing ORB ----------------------------------------------------------'

for n in xrange(d0, d1,1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = orb.calc_shift_and_rotation(img0, img1, maxMatches=N)
    print '\torb: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, orb.Nfms[-1])

data_orb = orb.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_orb['dts']) * 1000, ' ms'

print 'Doing BRISK ----------------------------------------------------------'
for n in xrange(d0, d1,1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = brisk.calc_shift_and_rotation(img0, img1, maxMatches=N)
    print '\tbrisk: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, brisk.Nfms[-1])
    
data_brisk = brisk.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_brisk['dts']) * 1000, ' ms'

print 'Doing USURF ----------------------------------------------------------'
for n in xrange(d0, d1,1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = usurf.calc_shift_and_rotation(img0, img1, maxMatches=N)
    print '\tusurf: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, usurf.Nfms[-1])
    
data_usurf = usurf.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_usurf['dts']) * 1000, ' ms'

print 'Doing SURF ----------------------------------------------------------'
for n in xrange(d0, d1,1):
    img1 = imgs[n,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

    Ax,Ay,theta = surf.calc_shift_and_rotation(img0, img1, maxMatches=N)
    print '\tsurf: {0:0.2f}, {1:0.2f}, {2:0.2f} N={3}'.format(Ax,Ay,theta, surf.Nfms[-1])

data_surf = surf.get_data() #get results as dictionary 
print 'Average time ', np.mean(data_surf['dts']) * 1000, ' ms'


#%%-----------------------------------------------------------------------------
#save results to hdf5 file
#-------------------------------------------------------------------------------
out = h5py.File('..\\results\\4.2 methods rotations.hdf5', 'a')
if 'window size {0}'.format(win[2:]) not in out:
    g = out.create_group('window size {0}'.format(win[2:]) )
else:
    g = out['window size {0}'.format(win[2:])]
    g.clear()

g['Dangles'] = Dangles
misc.save_dict_to_hdf5( data_orb, g, key='orb' )
misc.save_dict_to_hdf5( data_brisk, g, key='brisk' )
misc.save_dict_to_hdf5( data_usurf, g, key='usurf' )
misc.save_dict_to_hdf5( data_surf, g, key='surf' )

out.close()

#%%-----------------------------------------------------------------------------
# 
#-------------------------------------------------------------------------------
plt.figure()
plt.plot(Dangles[d0:d1], data_orb['thetas'], '.', label='ORB')
plt.plot(Dangles[d0:d1], data_brisk['thetas'], '.', label='BRISK')
plt.plot(Dangles[d0:d1], data_usurf['thetas'], '.', label='USURF')
plt.plot(Dangles[d0:d1], data_surf['thetas'], '.', label='SURF')