"""
Table timing/accuracy data
"""

#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------

import matplotlib.pyplot as plt
plt.ion()

from lib import plotting
from lib import misc
import h5py

#%%-----------------------------------------------------------------------------
# table data combined
#-------------------------------------------------------------------------------
data_in = h5py.File('..\\results\\4.1 methods translation.hdf5', 'r')
data_in_rot = h5py.File('..\\results\\4.2 methods rotations.hdf5', 'r')

print 'method\t\t\t Translation \t\t Rotation'
print 'method\t\t\t err Ax\t err Ay\t err theta\t Nma\t\t dt (ms)\t\t sigma theta\t\t Nma\t\tdt (ms)'
for win in (128,256,512):

    win_size = (win,win)
    
    g1 = data_in['window size {0}'.format(win_size) ]
    data_ncc1   = misc.load_dict_from_hdf5( g1, key='ncc' )
    data_orb1   = misc.load_dict_from_hdf5( g1, key='orb' )
    data_brisk1 = misc.load_dict_from_hdf5( g1, key='brisk' )
    data_usurf1 = misc.load_dict_from_hdf5( g1, key='usurf' )
    data_surf1  = misc.load_dict_from_hdf5( g1, key='surf' )
    
    Axs_ncc = data_ncc1['Axs']
    Ays_ncc = data_ncc1['Ays']

    
    g2 = data_in_rot['window size {0}'.format(win_size) ]
    Dangles = g2['Dangles'][:]

    data_orb2   = misc.load_dict_from_hdf5( g2, key='orb' )
    data_brisk2 = misc.load_dict_from_hdf5( g2, key='brisk' )
    data_usurf2 = misc.load_dict_from_hdf5( g2, key='usurf' )
    data_surf2  = misc.load_dict_from_hdf5( g2, key='surf' )
    
    ##USURF
    print 'USURF ({0},{1} px) \t&\t'.format(win_size[0], win_size[1]),
    
    #translation
    errx1 = (data_usurf1['Axs']-Axs_ncc).std()
    erry1 = (data_usurf1['Ays']- Ays_ncc).std()
    errt1 = (data_usurf1['thetas'] - 0).std()
    Nm1 = data_usurf1['Nfms'].mean()
    dt1 = data_usurf1['dts'].mean()*1000 #ms
    
    #rotations (remove nans)
    errt2 = data_usurf2['thetas']-Dangles
    Nm2 = data_usurf2['Nfms']
    dt2 = data_usurf2['dts']
    Nm2 = Nm2[np.isnan(errt2)==False].mean()
    dt2 = dt2[np.isnan(errt2)].mean()*1000 #ms
    errt2 = errt2[np.isnan(errt2) ==False].std()
    
    print '{0:0.2f} \t&\t {1:0.2f} \t&\t {2:0.2f} \t&\t {3:0.0f} \t&\t {4:0.1f} \t&\t {5:0.2f} \t&\t {6:0.0f} \t&\t {7:0.1f}\\\\'.format( errx1, erry1, errt1, Nm1, dt1, errt2, Nm2, dt2)
       
    ##SURF
    print 'SURF ({0},{1} px) \t&\t'.format(win_size[0], win_size[1]),
    
    #translation
    errx1 = (data_surf1['Axs']-Axs_ncc).std()
    erry1 = (data_surf1['Ays']- Ays_ncc).std()
    errt1 = (data_surf1['thetas'] - 0).std()
    Nm1 = data_surf1['Nfms'].mean()
    dt1 = data_surf1['dts'].mean()*1000 #ms
    
    #rotations (remove nans)
    errt2 = data_surf2['thetas']-Dangles
    Nm2 = data_surf2['Nfms']
    dt2 = data_surf2['dts']
    Nm2 = Nm2[np.isnan(errt2)==False].mean()
    dt2 = dt2[np.isnan(errt2)].mean()*1000 #ms
    errt2 = errt2[np.isnan(errt2) ==False].std()
    
    print '{0:0.2f} \t&\t {1:0.2f} \t&\t {2:0.2f} \t&\t {3:0.0f} \t&\t {4:0.1f} \t&\t {5:0.2f} \t&\t {6:0.0f} \t&\t {7:0.1f}\\\\'.format( errx1, erry1, errt1, Nm1, dt1, errt2, Nm2, dt2)
       
    ##ORB
    print 'ORB ({0},{1} px) \t&\t'.format(win_size[0], win_size[1]),
    
    #translation
    errx1 = (data_orb1['Axs']-Axs_ncc).std()
    erry1 = (data_orb1['Ays']- Ays_ncc).std()
    errt1 = (data_orb1['thetas'] - 0).std()
    Nm1 = data_orb1['Nfms'].mean()
    dt1 = data_orb1['dts'].mean()*1000 #ms
    
    #rotations (remove nans)
    errt2 = data_orb2['thetas']-Dangles
    Nm2 = data_orb2['Nfms']
    dt2 = data_orb2['dts']
    Nm2 = Nm2[np.isnan(errt2)==False].mean()
    dt2 = dt2[np.isnan(errt2)].mean()*1000 #ms
    errt2 = errt2[np.isnan(errt2) ==False].std()
    
    print '{0:0.2f} \t&\t {1:0.2f} \t&\t {2:0.2f} \t&\t {3:0.0f} \t&\t {4:0.1f} \t&\t {5:0.2f} \t&\t {6:0.0f} \t&\t {7:0.1f}\\\\'.format( errx1, erry1, errt1, Nm1, dt1, errt2, Nm2, dt2)
       
    ##BRISK
    print 'BRISK ({0},{1} px) \t&\t'.format(win_size[0], win_size[1]),
    
    #translation
    errx1 = (data_brisk1['Axs']-Axs_ncc).std()
    erry1 = (data_brisk1['Ays']- Ays_ncc).std()
    errt1 = (data_brisk1['thetas'] - 0).std()
    Nm1 = data_brisk1['Nfms'].mean()
    dt1 = data_brisk1['dts'].mean()*1000 #ms
    
    #rotations (remove nans)
    errt2 = data_brisk2['thetas']-Dangles
    Nm2 = data_brisk2['Nfms']
    dt2 = data_brisk2['dts']
    Nm2 = Nm2[np.isnan(errt2)==False].mean()
    dt2 = dt2[np.isnan(errt2)].mean()*1000 #ms
    errt2 = errt2[np.isnan(errt2) ==False].std()
    
    print '{0:0.2f} \t&\t {1:0.2f} \t&\t {2:0.2f} \t&\t {3:0.0f} \t&\t {4:0.1f} \t&\t {5:0.2f} \t&\t {6:0.0f} \t&\t {7:0.1f}\\\\'.format( errx1, erry1, errt1, Nm1, dt1, errt2, Nm2, dt2)
       
                 
data_in.close()
data_in_rot.close()