"""
Make example videos of showing translation corrected windows
"""
#%%-----------------------------------------------------------------------------
# imports
#-------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
plt.ion()
from matplotlib.patches import Ellipse
import matplotlib.animation as manimation

import lib.misc as misc
from lib.plotting import *
import h5py

#%%-----------------------------------------------------------------------------
# make example videos of translating windows
#-------------------------------------------------------------------------------
surface, Rz = 'horizontal milling',2.5
path = '..\\data\\dataset4 surface samples translation\\'

filename = '{0} Rz={1} um.hdf5' .format(surface, Rz)
print 'Loading data from :', path+filename
data = h5py.File(path+filename, 'r')
imgs = data['imgs']

#generate windows
rect0 = (0, 32, 128,128)
Ax,Ay,c, wins = misc.get_shifted_windows(imgs, rect0, ref_i)
data.close()

#setup writer
FFMpegWriter = manimation.writers['ffmpeg']
metadata = dict(title='Translating window test', artist='Matplotlib',
                comment='Translation corrected windows from data set')
writer = FFMpegWriter(fps=5, metadata=metadata)

#setup the figure to animate
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

fig = plt.figure(figsize=(3,3))
ax1 = fig.add_axes( (0.1,0.1,0.8,0.8))

im1 = ax1.imshow( wins[0], interpolation='nearest')
ax1.set_xticks([])#hide labels
ax1.set_yticks([])

ax1.set_title('Translation corrected window')
tx1 = ax1.text( 10,20, r'$A_x = 0.0$', fontsize=12, color='w')

#animation loop
with writer.saving(fig, "..\\supplementary_data\\5.1 translating_windows.mp4", dpi=600):
    for n, win in enumerate(wins):
        im1.set_data(win)
        tx1.set_text(r'$A_x = {0:0.1f}$'.format(Ax[n]))
        writer.grab_frame()

#%%-----------------------------------------------------------------------------
# make example videos of rotating windows
#-------------------------------------------------------------------------------
surface, Rz = 'horizontal milling',2.5
path = '..\\data\\dataset5 surface samples rotation\\'

filename = '{0} Rz={1} um.hdf5' .format(surface, Rz)
print 'Loading data from :', path+filename
data = h5py.File(path+filename, 'r')
imgs = data['imgs']
angs = data['angs'][:]

#generate windows
ref_i = 0
centre = (490, 625)
radius = 64
pad = 32
mask = False
Ax, Ay, Dangs, c, wins, wins_ = misc.get_rotated_windows( imgs, angs, centre=centre, radius=radius, pad=pad, ref_i=ref_i, mask=mask)
data.close()

#setup writer
FFMpegWriter = manimation.writers['ffmpeg']
metadata = dict(title='Rotating window test', artist='Matplotlib',
                comment='Centered and Centered and rotated windows from data set')
writer = FFMpegWriter(fps=5, metadata=metadata)

#setup the figure to animate
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

fig = plt.figure(figsize=(6,3))
ax1 = fig.add_axes( (0.05,0.1,0.4,0.8))
ax2 = fig.add_axes( (0.55,0.1,0.4,0.8))

im1 = ax1.imshow( wins[0], interpolation='nearest')
ax1.set_xticks([])#hide labels
ax1.set_yticks([])
ax1.set_title('Centered window (original data)')
tx1 = ax1.text( 20,40, r'$\theta =-180$', fontsize=12, color='w')

im2 = ax2.imshow( wins_[0], interpolation='nearest')
ax2.set_xticks([])#hide labels
ax2.set_yticks([])
ax2.set_title(r'Centered and rotated (to {0}$\degree$) window'.format(angs[ref_i]))

#animation loop
with writer.saving(fig, "..\\supplementary_data\\5.1 rotating_windows.mp4", dpi=600):
    for n, (win, win_) in enumerate(zip(wins, wins_)):
        im1.set_data(win)
        im2.set_data(win_)
        tx1.set_text(r'$\theta ={0}$'.format(Dangs[n]))
        writer.grab_frame()
        
