"""
Test of feature detectors using opencv used to generate the times shown in table 1
"""
#%%-----------------------------------------------------------------------------
# imports and load data
#-------------------------------------------------------------------------------
import numpy as np
import time
import h5py

from lib import detectors
from lib import misc

f = h5py.File('..\\data\\dataset2 random positions.hdf5', 'r')
imgs = f['imgs']



win = (0,0,512,512)          #window size to use (x0,y0,w,h)
#win = (128,128,256,256)     #window size to use (x0,y0,w,h)
#win = (192,192,128,128)     #window size to use (x0,y0,w,h)

imgs = imgs[:,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

#%%-----------------------------------------------------------------------------
# #Harris
#-------------------------------------------------------------------------------
print 'Doing: Harris corners'

detector = detectors.Harris_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_harris = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'

#%%-----------------------------------------------------------------------------
# GFTT / Shi-Tomasi
#-------------------------------------------------------------------------------
print 'Doing: GFTT'
detector = detectors.GFTT_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_gftt = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'


#%%-----------------------------------------------------------------------------
# Fast detector
#-------------------------------------------------------------------------------
print 'Doing: FAST'

detector = detectors.FAST_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_fast = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'


#%%-----------------------------------------------------------------------------
# AGAST detector
#-------------------------------------------------------------------------------
print 'Doing: AGAST'

detector = detectors.AGAST_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_agast = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'

#%%-----------------------------------------------------------------------------
# USURF - Difference of Gaussians detector
#-------------------------------------------------------------------------------
print 'Doing: USURF'

detector = detectors.USURF_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_usurf = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'

#%%-----------------------------------------------------------------------------
# ORB - orientated FAST and BRIEF descriptor
#-------------------------------------------------------------------------------
print 'Doing: ORB'

detector = detectors.ORB_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_orb = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'


#%%-----------------------------------------------------------------------------
# BRISK (orientated fast/agast like detector)
#-------------------------------------------------------------------------------
print 'Doing: BRISK'

detector = detectors.BRISK_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_brisk = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'


#%%-----------------------------------------------------------------------------
# SURF - orientated Difference of Gaussians detector
#-------------------------------------------------------------------------------
print 'Doing: SURF'

detector = detectors.SURF_Detector()
for img in imgs:
    kps = detector.find_features(img)

N = np.mean(detector.N)
Nmin = np.min(detector.N)
Nmax = np.max(detector.N)

dt = np.mean(detector.dt)
dt_min = np.min(detector.dt)
dt_max = np.max(detector.dt)

data_surf = detector.get_data()

print 'Features found (min, mean, max): ',Nmin, N, Nmax
print 'Time per frame (min, mean, max): ',dt_min, dt, dt_max
print '-------------------------------------------------------------------'

#%%-----------------------------------------------------------------------------
# save results to file
#-------------------------------------------------------------------------------
out = h5py.File('..\\results\\5.2 detectors.hdf5', 'a')
if 'window size {0}'.format(win[2:]) not in out:
    g = out.create_group('window size {0}'.format(win[2:]) )
else:
    g = out['window size {0}'.format(win[2:])]
    g.clear()

misc.save_dict_to_hdf5( data_harris, g, 'harris')
misc.save_dict_to_hdf5( data_gftt, g, 'gftt')
misc.save_dict_to_hdf5( data_fast, g, 'fast')
misc.save_dict_to_hdf5( data_agast, g, 'agast')
misc.save_dict_to_hdf5( data_usurf, g, 'usurf')
misc.save_dict_to_hdf5( data_orb, g, 'orb')
misc.save_dict_to_hdf5( data_brisk, g, 'brisk')
misc.save_dict_to_hdf5( data_surf, g, 'surf')

out.close()

#%%-----------------------------------------------------------------------------
# table data
#-------------------------------------------------------------------------------
data_in = h5py.File('..\\results\\5.2 detectors.hdf5', 'r')

print 'method\t\t\t dt(128)\t dt(256)\t dt (512)\t N(128)\t N(256)\t N(512) '

for method in ['harris', 'gftt', 'fast', 'agast', 'usurf', 'orb', 'brisk', 'surf']:

    print '{0}'.format(method),

    #Do the timings
    for win in (128,256,512):

        win_size = (win,win)
        g = data_in['window size {0}'.format(win_size) ]

        data   = misc.load_dict_from_hdf5( g, key=method )
        #x1000 to ms
        print '\t&\t{0:0.2f}'.format( np.mean(data['dt'])*1000 ), 
        
    #Do the Ns
    for win in (128,256,512):

        win_size = (win,win)
        g = data_in['window size {0}'.format(win_size) ]

        data   = misc.load_dict_from_hdf5( g, key=method )
        print '\t&\t{0:0.0f}'.format( np.mean(data['N']) ),
    print '\\\\'
data_in.close()
