"""
Figure generation Detector translation robustness
"""
#%%-----------------------------------------------------------------------------
# imports
#-------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
plt.ion()

from mpl_toolkits.axes_grid1 import ImageGrid

from lib.plotting import * 
from lib import misc

import textwrap

import h5py

#%%-----------------------------------------------------------------------------
# setup samples names
#-------------------------------------------------------------------------------
#surface preparation methods
surfaces = ( ('horizontal milling', (2.5,16,50)),
            ('flat lapping reaming', (0.55,1.6,10)),
            ('vertical milling', (2.5,16,50)),
            ('turning', (2.5,16,50)),
            ('grinding', (0.55,1.6,10))  )

#%%-----------------------------------------------------------------------------
# Figure Translation R vs C  - full page all samples
#-------------------------------------------------------------------------------
data = h5py.File('..\\results\\5.3 detector_translation_robustness.hdf5', 'r')

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

fig = plt.figure( 'Translation: Detector robustness vs C 2 ', figsize=(8,11))

ag = axis_grid( fig, rect=(0.05,0.05,0.9,0.95), rowscols=(5,3), pad=(0.02,0.02))
n=0
for surf_treatment, Rzs in surfaces:
    print surf_treatment, 
    for Rz in Rzs:
        print 'Rz=',Rz,
        
        #load processed results
        surface = '{0} Rz={1} um'.format(surf_treatment, Rz)
        g = data[surface]
        c = g['c']
        
        res_fast = misc.Structure( misc.load_dict_from_hdf5(g, 'FAST')) 
        res_agast = misc.Structure( misc.load_dict_from_hdf5(g, 'AGAST')) 
        res_harris = misc.Structure( misc.load_dict_from_hdf5(g, 'Harris')) 
        res_dog = misc.Structure( misc.load_dict_from_hdf5(g, 'DoG')) 
        
        #plot
        ax = ag[n]
        ax.plot(c, res_harris.R, '^', label='Harris', color=cmap(2), ms=3)
        ax.plot(c, res_agast.R, 'x', label='AGAST', color=cmap(1), ms=3)
        ax.plot(c, res_fast.R, '.', label='FAST', color=cmap(0), ms=3)
        ax.plot(c, res_dog.R, '*', label='DoG/USURF', color=cmap(3), ms=3)
        ax.set_xlim(1,0.25)
        ax.set_ylim(0,1)
        ax.text( 0.8, 0.9,surf_treatment)
        ax.text( 0.8, 0.8,'$R_z={0} \mu m$'.format(Rz))
        
        ax.set_xlabel('Correlation Coefficent')
        ax.set_ylabel(r'Robustness $\frac{N_{m}}{N_{0}} $')
        n+=1
        
    print 'Done'
    
ax = ag[0]
l =  ax.legend(loc=(0.45,0.4), labelspacing=0.1, frameon=True, handletextpad=0.1, borderpad=0.2 )
show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)

data.close()

fig.savefig('..\\supplementary_data\\5.3 detector_translation_robustness_vs_c.pdf', dpi=600)

#%%-----------------------------------------------------------------------------
# Figure Translation R vs Ax - full page all samples
#-------------------------------------------------------------------------------
#main results
data = h5py.File('..\\results\\5.3 detector_translation_robustness.hdf5', 'r')

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

fig = plt.figure( 'Translation: Detector robustness vs Ax', figsize=(8,11))
ag = axis_grid( fig, rect=(0.05,0.05,0.9,0.95), rowscols=(5,3), pad=(0.03,0.03))
n=0
for surf_treatment,Rzs in surfaces:
    print surf_treatment, 
    for Rz in Rzs:
        print 'Rz=',Rz,
        
        #load processed results
        surface = '{0} Rz={1} um'.format(surf_treatment, Rz)
        g = data[surface]
        Ax = g['Ax']
        
        res_fast = misc.Structure( misc.load_dict_from_hdf5(g, 'FAST')) 
        res_agast = misc.Structure( misc.load_dict_from_hdf5(g, 'AGAST')) 
        res_harris = misc.Structure( misc.load_dict_from_hdf5(g, 'Harris')) 
        res_dog = misc.Structure( misc.load_dict_from_hdf5(g, 'DoG')) 
        
        #plot
        ax = ag[n]
        ax.set_aspect('auto')
        ax.plot(Ax, res_harris.R, '^', label='Harris', color=cmap(2), ms=3)
        ax.plot(Ax, res_agast.R, 'x', label='AGAST', color=cmap(1), ms=3)
        ax.plot(Ax, res_fast.R, '.', label='FAST', color=cmap(0), ms=3)
        ax.plot(Ax, res_dog.R, '*', label='DoG/USURF', color=cmap(3), ms=3)
        ax.set_xlim(0,1200)
        ax.set_ylim(0,1)
        ax.text( 0.25, 0.9,surf_treatment, transform=ax.transAxes)
        ax.text( 0.25, 0.8,'$R_z={0} \mu m$'.format(Rz), transform=ax.transAxes)
        
        ax.set_xlabel('Speckle translation [px]')
        ax.set_ylabel(r'Robustness $\frac{N_{m}}{N_{0}} $')
        n+=1
        
    print 'Done'
    
ax = ag[0]
l =  ax.legend(loc=(0.45,0.4), labelspacing=0.1, frameon=True, handletextpad=0.1, borderpad=0.2 )
show_outside_axis_only(ag, labels = False, ticklabels=False, ticks=True)

fig.savefig('..\\supplementary_data\\5.3 detector_translation_robustness_vs_Ax.pdf', dpi=600)
data.close()



#%%-----------------------------------------------------------------------------
# Figure: 4 samples (R vs Ax) - journal figure
#-------------------------------------------------------------------------------
data = h5py.File('..\\results\\5.3 detector_translation_robustness.hdf5', 'r')

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

# 1x4
f = plt.figure(figsize=(6.7,2.0))
ag = axis_grid( f, (0.05,0.25,0.9,0.65), rowscols=(1,4), pad=(0.05,0.1))
filename = '..\\figures\\detector_translation_robustness_vs_Ax_(1x4).pdf'
titles=[]
for surf_treatment,Rz in samples:
    title = surf_treatment.capitalize()+'\n$R_z={0} \mu m$'.format(Rz)
    titles.append(title)

#2x2
#f = plt.figure(figsize=(3.25,3.25))
#ag = axis_grid( f, (0.12,0.15,0.84,0.86), rowscols=(2,2), pad=(0.06,0.1))
#filename = '..\\figures\\detector_translation_robustness_vs_Ax_(2x2).pdf'
#titles=('a)', 'b)', 'c)', 'd)')
#

samples = ( ('turning', 50),
            ('horizontal milling', 16),
            ('flat lapping reaming', 1.6),
            ('vertical milling', 16),
            )
n=0
for surf_treatment,Rz in samples:
    print surf_treatment, Rz 

    #load processed results
    surface = '{0} Rz={1} um'.format(surf_treatment, Rz)
    g = data[surface]
    Ax = g['Ax']
    
    res_fast = misc.Structure( misc.load_dict_from_hdf5(g, 'FAST')) 
    res_agast = misc.Structure( misc.load_dict_from_hdf5(g, 'AGAST')) 
    res_harris = misc.Structure( misc.load_dict_from_hdf5(g, 'Harris')) 
    res_dog = misc.Structure( misc.load_dict_from_hdf5(g, 'DoG')) 
    
    #plot
    ax = ag[n]
    ax.set_aspect('auto')
    ax.plot(Ax, res_harris.R, '^', label='Harris', color=cmap(2), ms=3)
    ax.plot(Ax, res_agast.R, 'x', label='AGAST', color=cmap(1), ms=3)
    ax.plot(Ax, res_fast.R, '.', label='FAST', color=cmap(0), ms=3)
    ax.plot(Ax, res_dog.R, '*', label='DoG/USURF', color=cmap(3), ms=3)
    ax.set_xlim(0,1200)
    ax.set_ylim(0,1)
    #ax.text( 0.25, 0.9,surf_treatment, transform=ax.transAxes)
    #ax.text( 0.25, 0.8,'$R_z={0} \mu m$'.format(Rz), transform=ax.transAxes)
    #ax.set_title(surf_treatment.capitalize()+'\n$R_z={0} \mu m$'.format(Rz), fontsize=8)
    ax.set_title(titles[n], fontsize=8)
    ax.set_xlabel('Speckle\ntranslation [px]')
    ax.set_ylabel(r'Robustness $\frac{N_{m}}{N_{0}} $')
    ax.set_yticks( (0,0.2,0.4,0.6,0.8,1.0))
    n+=1
        
    print 'Done'
    
ax = ag[0]
l =  ax.legend(loc=(0.2,0.48), labelspacing=0.1, frameon=False, handletextpad=0.1, borderpad=0 )
show_outside_axis_only(ag, labels=False, ticklabels=False, ticks=True)

f.savefig(filename, dpi=600)
data.close()

#%%-----------------------------------------------------------------------------
# Figure: 4 samples (R vs C) - journal figure
#-------------------------------------------------------------------------------
data = h5py.File('..\\results\\5.3 detector_translation_robustness.hdf5', 'r')

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

# 1x4
f = plt.figure(figsize=(6.7,2.0))
ag = axis_grid( f, (0.05,0.25,0.9,0.65), rowscols=(1,4), pad=(0.05,0.1))
filename = '..\\figures\\detector_translation_robustness_vs_c_(1x4).pdf'
titles=[]
for surf_treatment,Rz in samples:
    title = surf_treatment.capitalize()+'\n$R_z={0} \mu m$'.format(Rz)
    titles.append(title)

#2x2
#f = plt.figure(figsize=(3.25,3.25))
#ag = axis_grid( f, (0.12,0.15,0.84,0.86), rowscols=(2,2), pad=(0.06,0.1))
#filename = '..\\figures\\detector_translation_robustness_vs_c_(2x2).pdf'
#titles=('a)', 'b)', 'c)', 'd)')
#

samples = ( ('turning', 50),
            ('horizontal milling', 16),
            ('flat lapping reaming', 1.6),
            ('vertical milling', 16),
            )
n=0
for surf_treatment,Rz in samples:
    print surf_treatment, Rz 

    #load processed results
    surface = '{0} Rz={1} um'.format(surf_treatment, Rz)
    g = data[surface]
    c = g['c']
    
    res_fast = misc.Structure( misc.load_dict_from_hdf5(g, 'FAST')) 
    res_agast = misc.Structure( misc.load_dict_from_hdf5(g, 'AGAST')) 
    res_harris = misc.Structure( misc.load_dict_from_hdf5(g, 'Harris')) 
    res_dog = misc.Structure( misc.load_dict_from_hdf5(g, 'DoG')) 
    
    #plot
    ax = ag[n]
    ax.plot(c, res_harris.R, '^', label='Harris', color=cmap(2), ms=3)
    ax.plot(c, res_agast.R, 'x', label='AGAST', color=cmap(1), ms=3)
    ax.plot(c, res_fast.R, '.', label='FAST', color=cmap(0), ms=3)
    ax.plot(c, res_dog.R, '*', label='DoG/USURF', color=cmap(3), ms=3)
    ax.set_xlim(1,0.0)
    ax.set_ylim(0,1)
    #ax.text( 0.8, 0.9,surf_treatment)
    #ax.text( 0.8, 0.8,'$R_z={0} \mu m$'.format(Rz))
    #ax.set_title(surf_treatment.capitalize()+'\n$R_z={0} \mu m$'.format(Rz), fontsize=8)
    ax.set_title(titles[n], fontsize=8)

    ax.set_xlabel('Correlation\ncoefficent')
    ax.set_ylabel(r'Robustness $\frac{N_{m}}{N_{0}} $')
    ax.set_yticks( (0,0.2,0.4,0.6,0.8,1.0))
    n+=1
    print 'Done'
    
ax = ag[0]
l =  ax.legend(loc=(0.2,0.48), labelspacing=0.1, frameon=False, handletextpad=0.1, borderpad=0 )
show_outside_axis_only(ag, labels=False, ticklabels=False, ticks=True)

f.savefig(filename, dpi=600)
data.close()
