"""
Figure showing rotating speckle patterns
"""
#%%-----------------------------------------------------------------------------
# imports
#-------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
plt.ion()
from matplotlib.patches import Ellipse
import matplotlib.animation as manimation

import lib.misc as misc
from lib.plotting import *
import h5py
#%%-----------------------------------------------------------------------------
# Smaller windows for figures/video
#-------------------------------------------------------------------------------
surface, Rz = 'horizontal milling',2.5
path = '..\\data\\dataset5 surface samples rotation\\'

filename = '{0} Rz={1} um.hdf5' .format(surface, Rz)
print 'Loading data from :', path+filename
data = h5py.File(path+filename, 'r')
imgs = data['imgs']
angs = data['angs'][:]


ref_i = 0
centre = (490, 625)
radius = 64
pad = 32
mask = False

#generate windows
Ax, Ay, Dangs, c, wins, wins_ = misc.get_rotated_windows( imgs, angs, centre=centre, radius=radius, pad=pad, ref_i=ref_i, mask=mask)
data.close()


#%%-----------------------------------------------------------------------------
# figure showing rotated windows
#-------------------------------------------------------------------------------

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)


f = plt.figure(figsize=(6.7,3.0))
ag = axis_grid( f, (0.05,0.05,0.9,0.95), rowscols=(2,5), pad=(0.01,0.1))
plt.jet()

angs = [0,10,20,30,40, 60,90,120,150,180]
for n,ang in enumerate(angs):
    i = Dangs.tolist().index(ang)
    ax = ag[n]
    im = ax.imshow(wins_[i], interpolation='nearest')
    im.set_clim(10,216)
    ax.set_title('{0}$\degree$'.format(ang), fontsize=8)
    ax.set_xlim(pad, wins.shape[2]-pad)
    ax.set_ylim(pad, wins.shape[1]-pad)
    ax.set_xticks([])
    ax.set_yticks([])
    
    # set of speckles with same pattern
    l = plt.Line2D([57,100,128, 93], [108,64,97,145], ls='--', color='w', alpha=0.8)
    ax.add_line(l)
    t = ax.text(68,105, 'A', color='w', fontsize=8)    
    
    #cluster of 3/4 speckles that change
    t = ax.text(139,118, 'B', color='w', fontsize=8)    
    e = Ellipse( (142,99), width=40, height=25, angle=15, color='w', ls='--', fill=None)
    ax.add_patch(e)

f.savefig('..\\figures\\pattern_rotations.pdf', dpi=600, transparent=True)