"""
Test of feature descriptor and matching calculation times
"""
#%%-----------------------------------------------------------------------------
# imports and load data
#-------------------------------------------------------------------------------
import numpy as np
import time
import h5py
import cv2

from lib import detectors
from lib import descriptors
from lib import misc

f = h5py.File('..\\data\\dataset2 random positions.hdf5', 'r')
imgs = f['imgs']

#image size to use
win = (0,0,512,512)          #window size to use (x0,y0,w,h)
#win = (128,128,256,256)     #window size to use (x0,y0,w,h)
#win = (192,192,128,128)     #window size to use (x0,y0,w,h)
imgs = imgs[:,win[1]:win[1]+win[3], win[0]:win[0]+win[2]]

out = h5py.File('..\\results\\5.4 descriptors.hdf5', 'a')

#%%-----------------------------------------------------------------------------
# Find features using the ORB detector (for orientation)
#-------------------------------------------------------------------------------
detector = detectors.ORB_Detector()
kps_list = []
for img in imgs:
    kps = detector.find_features(img)
    kps_list.append(kps)
    
    
#%%-----------------------------------------------------------------------------
# USURF
#-------------------------------------------------------------------------------
usurf1 = descriptors.SURF( upright=True, extended=False)
usurf2 = descriptors.SURF( upright=True, extended=True)

for n in range(0, len(kps_list)):
    kps,des = usurf1.compute_descriptors(imgs[n], kps_list[n])
    kps,des = usurf2.compute_descriptors(imgs[n], kps_list[n])

#get data
data_usurf1 =usurf1.get_data()
data_usurf2 = usurf2.get_data()


misc.save_dict_to_hdf5( data_usurf1, out, key=usurf1.method )
misc.save_dict_to_hdf5( data_usurf2, out, key=usurf2.method )


#%%-----------------------------------------------------------------------------
# SURF
#-------------------------------------------------------------------------------
surf1 = descriptors.SURF( upright=False, extended=False)
surf2 = descriptors.SURF( upright=False, extended=True)

for n in range(0, len(kps_list)):
    kps,des = surf1.compute_descriptors(imgs[n], kps_list[n])
    kps,des = surf2.compute_descriptors(imgs[n], kps_list[n])

#get data
data_surf1 = surf1.get_data()
data_surf2 = surf2.get_data()

misc.save_dict_to_hdf5( data_surf1, out, key=surf1.method )
misc.save_dict_to_hdf5( data_surf2, out, key=surf2.method )

#%%-----------------------------------------------------------------------------
# ORB
#-------------------------------------------------------------------------------
orb16 = descriptors.ORB(patchSize=16)
orb32 = descriptors.ORB(patchSize=32)
orb48 = descriptors.ORB(patchSize=48)

for n in range(0, len(kps_list)):
    kps,des = orb16.compute_descriptors(imgs[n], kps_list[n])
    kps,des = orb32.compute_descriptors(imgs[n], kps_list[n])
    kps,des = orb48.compute_descriptors(imgs[n], kps_list[n])

#get data
data_orb16 = orb16.get_data()
data_orb32 = orb32.get_data()
data_orb48 = orb48.get_data()

misc.save_dict_to_hdf5( data_orb16, out, key=orb16.method )
misc.save_dict_to_hdf5( data_orb32, out, key=orb32.method )
misc.save_dict_to_hdf5( data_orb48, out, key=orb48.method )

#%%-----------------------------------------------------------------------------
# BRISK
#-------------------------------------------------------------------------------
brisk16 = descriptors.BRISK(patchSize=16)
brisk32 = descriptors.BRISK(patchSize=32)
brisk48 = descriptors.BRISK(patchSize=48)

for n in range(0, len(kps_list)):
    kps,des = brisk16.compute_descriptors(imgs[n], kps_list[n])
    kps,des = brisk32.compute_descriptors(imgs[n], kps_list[n])
    kps,des = brisk48.compute_descriptors(imgs[n], kps_list[n])

#get data
data_brisk16 = brisk16.get_data()
data_brisk32 = brisk32.get_data()
data_brisk48 = brisk48.get_data()

misc.save_dict_to_hdf5( data_brisk16, out, key=brisk16.method )
misc.save_dict_to_hdf5( data_brisk32, out, key=brisk32.method )
misc.save_dict_to_hdf5( data_brisk48, out, key=brisk48.method )

out.close()

#%%-----------------------------------------------------------------------------
# Timing table
#-------------------------------------------------------------------------------
out = h5py.File('..\\results\\5.4 descriptors.hdf5', 'a')
print 'method  \t\t&\t\t ms/1k features.\\\\'
for key in out:
    data = out[key]
    per_1kfeatures =  ( np.sum(data['dt'])/np.sum(data['N']))  * 1000 * 1000 
    print '{0} \t&\t {1:0.1f} \\\\'.format(data.attrs['method'] , per_1kfeatures)
out.close()

