"""
Figure: histogram of good/bad/failed matches when looking at orientation calc.
"""
#%%-----------------------------------------------------------------------------
# setup
#-------------------------------------------------------------------------------

import lib.detectors as detectors
from lib import descriptors 
import lib.misc as misc
import lib.plotting as plotting
import h5py
import numpy as np
import cv2 as cv
import matplotlib.pyplot as plt
plt.ion()

#%%-----------------------------------------------------------------------------
# figure histogram all methods
#-------------------------------------------------------------------------------
data_path = '..\\results\\5.5.1 orientation_histogram.hdf5'
data = h5py.File(data_path, 'a')  

#no point in plotting usurf as no orientation angle!
to_plot = ( 'SURF (length 64 descriptor)',   
            'ORB (patchSize = 32)',
            'BRISK (patchSize = 32)', 
            )   
            

font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)

cmap = plt.get_cmap('tab10')

# 1x3
f = plt.figure(figsize=(6.7,2.0))
ag = plotting.axis_grid( f, (0.05,0.25,0.9,0.65), rowscols=(1,3), pad=(0.05,0.1))
filename = '..\\figures\\orientation_histogram_(1x3).pdf'
titles=['SURF', 'ORB', 'BRISK']


bins = np.linspace(-180,180,180)

for n, method in enumerate(to_plot):
    res  = data[ method ]

    good  = res['good']
    bad   = res['bad']
    failed = res['failed']
    ugly = res['ugly'] #np.hstack((bad,failed))

    ax = ag[n]
    ax.hist(ugly, bins, color='red', label='Bad')
    ax.hist(failed, bins, color='orange', label='Failed')
    ax.hist(good, bins, color='green', label='Good')
    ax.set_ylim(0,1500)
    ax.set_xlim((-180,180))
    
    ax.set_ylabel('Number\nof features')
    #ax.set_xlabel('Difference in calculated\nfeature orientation [$^o$]')
    ax.set_xlabel(r'$\Delta \theta_{orient}$ [$^o$]')
    ax.set_title(titles[n], fontsize=8)

ax = ag[0]
l =  ax.legend(loc=(0.04,0.5), labelspacing=0.05, frameon=False, handletextpad=0.1, borderpad=0, handlelength=1 )
plotting.show_outside_axis_only(ag, labels=False, ticklabels=False, ticks=True)


f.savefig(filename, dpi=600)

data.close()