"""
Descriptor descrimination - How does the distance of known matched speckle vary
compared to the distane of a totally random match.

Uses distance from random speckle for the uncorrelated band
 """
 #%%-----------------------------------------------------------------------------
# imports and load data
#-------------------------------------------------------------------------------
import numpy as np
import time
import h5py

from lib import detectors
from lib import descriptors
from lib import misc
from lib import plotting

#distance calculations
from lib.descriptors import L1_norm, L2_norm, hamming

import matplotlib.pyplot as plt
import cv2 as cv

#surface preparation methods
surfaces = ( ('horizontal milling', (2.5,16,50)),
            ('flat lapping reaming', (0.55,1.6,10)),
            ('vertical milling', (2.5,16,50)),
            ('turning', (2.5,16,50)),
            ('grinding', (0.55,1.6,10))  )

#select
surface, Rzs = surfaces[0]
Rz = Rzs[1]

#%%-----------------------------------------------------------------------------
# set descriptors to test
#-------------------------------------------------------------------------------
#detectors
orb_detector = detectors.ORB_Detector()      #oFAST
brisk_detector = detectors.BRISK_Detector()     #oFAST
surf_detector = detectors.SURF_Detector()     #oDoG
usurf_detector = detectors.USURF_Detector()     #DoG

#descriptors
usurf1 = descriptors.SURF( upright=True, extended=False)
usurf2 = descriptors.SURF( upright=True, extended=True)
surf1 = descriptors.SURF( upright=False, extended=False)
surf2 = descriptors.SURF( upright=False, extended=True)
orb16 = descriptors.ORB(patchSize=16)
orb32 = descriptors.ORB(patchSize=32)
orb48 = descriptors.ORB(patchSize=48)
orb64 = descriptors.ORB(patchSize=64)
brisk16 = descriptors.BRISK(patchSize=16)
brisk32 = descriptors.BRISK(patchSize=32)
brisk48 = descriptors.BRISK(patchSize=48)
brisk64 = descriptors.BRISK(patchSize=64)

# detector, descriptor, distance_func = L1_norm/L2_norm/hamming
to_test = ( (usurf_detector, usurf1, L1_norm), (usurf_detector, usurf2, L1_norm),   #0,1
            (surf_detector, surf1, L1_norm), (surf_detector, surf2, L1_norm),     #2,3
            (orb_detector, orb16, hamming), (orb_detector, orb32, hamming), (orb_detector, orb48, hamming), (orb_detector, orb64, hamming), #4,5,6,7
            (brisk_detector, brisk16, hamming), (brisk_detector, brisk32, hamming), (brisk_detector, brisk48, hamming), (brisk_detector, brisk64, hamming),  #8,9,10,11
            )
            
#to_test = ( (brisk_detector, brisk16, hamming), (brisk_detector, brisk32, hamming),)

#%%-----------------------------------------------------------------------------
# translated data set
#-------------------------------------------------------------------------------
path = '..\\data\\dataset4 surface samples translation\\'
surface_name = '{0} Rz={1} um'.format(surface, Rz)
print 'Loading data from :', path+surface

data = h5py.File(path+surface_name+'.hdf5', 'r')
imgs = data['imgs']

#store results to...
out_file = h5py.File('..\\results\\5.5.3 descrimination_translation.hdf5', 'a')
if surface_name in out_file:
    out = out_file[surface_name]
else:
    out = out_file.create_group(surface_name)

# get shifted windows
#speckles move +Ax, use pixel border to allow slight Ay changes
rect0 = (0, 32, 192 ,960)

ref_i = 0
Ax,Ay,c, wins = misc.get_shifted_windows(imgs[:-2], rect0, ref_i)       #window too big for later shifts
N=1000  #number of random matches to compute

for detector, descriptor, distance_func in  to_test:

    print 'Doing', descriptor.method, detector.method, distance_func

    ## Uncorrelated speckles
    #get two uncorreleted speckle patterns - no good matches! 
    win0 = imgs[0][:, 0:640]
    win1 = imgs[0][:, 640:]

    #find keypoints
    kps0 = detector.find_features(win0)
    kps1 = detector.find_features(win1)
    
    #compute descriptors
    kps0_d, Ds0 = descriptor.compute_descriptors(win0, kps0)
    kps1_d, Ds1 = descriptor.compute_descriptors(win1, kps1)
    print 'Number of KPs', len(kps0_d), len(kps1_d)

    #ensure same number and mixed up (incase compute descriptors orders the keypoints by some score)
    #N = min(len(kps0_d),len(kps1_d))
    i0 = np.random.randint(0, len(kps0_d), N)
    i1 = np.random.randint(0, len(kps1_d), N)
    Ds0 = Ds0[i0]
    Ds1 = Ds1[i1]
    
    #find distances between keypoint pairs
    d_uncorrelated = distance_func(Ds0, Ds1)
    
    ## Correlated speckles from window to window as it translates
    #find keypoints in first window
    win0 = wins[ref_i]
    kps0 = detector.find_features(win0)

    #compute descriptors in first window
    kps0_d, Ds0 = descriptor.compute_descriptors(win0, kps0)
    
    #loop over all windows (varying translation)
    d_mean = []
    d_std  = []
    d_min  = []
    d_max  = []
    for win in wins:
        
        #compute descriptors at same points in new window
        kps1_d, Ds1 = descriptor.compute_descriptors(win, kps0)

        #find distances between keypoint pairs
        d_correlated = distance_func(Ds0, Ds1)
    
        #find  statistics
        d_mean.append(d_correlated.mean())
        d_std.append(d_correlated.std())
        d_min.append(d_correlated.min())
        d_max.append(d_correlated.max())
        
    d_mean = np.array(d_mean)
    d_std = np.array(d_std)
    d_min = np.array(d_min)
    d_max = np.array(d_max)
    
    
    #store results
    res = { 'd_uncorrelated':d_uncorrelated,
            'd_mean': d_mean,
            'd_std':d_std,
            'd_min':d_min,
            'd_max':d_max, 
            'Ax':Ax,
            'Ay':Ay,
            'c':c, }

    misc.save_dict_to_hdf5(res , out, descriptor.method)
    
    #plot
    #plt.figure(filename)
    #l = plt.semilogy(Ax , d_mean, '.', label=descriptor.method)[0]
    #plt.hlines(d_uncorrelated.mean(), 0, 1100, colors=l.get_color())

#close output file
out_file.close()
#close input file
data.close()


#%%-----------------------------------------------------------------------------
# Rotating data set
#-------------------------------------------------------------------------------
path = '..\\data\\dataset5 surface samples rotation\\'
surface_name = '{0} Rz={1} um'.format(surface, Rz)
print 'Loading data from :', path+surface_name

data = h5py.File(path+surface_name+'.hdf5', 'r')
imgs = data['imgs']
angs = data['angs']

#store results to...
out_file = h5py.File('..\\results\\descrimination_rotation.hdf5', 'a')
if surface in out_file:
    out = out_file[surface]
else:
    out = out_file.create_group(surface)

# get windows
centre=(490,625) 
ref_i = 0              #reference to this image
radius = 320           #size of windows to extract
pad = 64               #size of border padding
Ax, Ay, Dangs, c, wins, wins_ = misc.get_rotated_windows( imgs, angs, centre=centre, radius=radius, pad=pad, ref_i=ref_i, mask=False)
#wins are centered window but still rotated originals
#wins_ are centered and rotated to same orientation for testing comparision

N=1000  #number of random matches to compute
    
for detector, descriptor, distance_func in  to_test:

    print 'Doing', descriptor.method, detector.method, distance_func

    ## Uncorrelated speckles
    #get two uncorreleted speckle patterns - no good matches! 
    win0 = imgs[0][:, 0:640]
    win1 = imgs[0][:, 640:]
    
    #find keypoints
    kps0 = detector.find_features(win0)
    kps1 = detector.find_features(win1)
    
    #compute descriptors
    kps0_d, Ds0 = descriptor.compute_descriptors(win0, kps0)
    kps1_d, Ds1 = descriptor.compute_descriptors(win1, kps1)
    print 'Number of KPs', len(kps0_d), len(kps1_d)

    #ensure same number and mixed up (incase compute descriptors orders the keypoints by some score)
    #N = min(len(kps0_d),len(kps1_d))
    i0 = np.random.randint(0, len(kps0_d), N)
    i1 = np.random.randint(0, len(kps1_d), N)
    Ds0 = Ds0[i0]
    Ds1 = Ds1[i1]
    
    #find distances between keypoint pairs
    d_uncorrelated = distance_func(Ds0, Ds1)
    
    ## Correlated speckles from window to window as it translates
    #find keypoints in first window
    win0 = wins[ref_i]
    kps0 = detector.find_features(win0)
    Cy,Cx  = np.array(win0.shape)//2 #center or rotation
    kps0 = misc.filter_kps( kps0, centre=(Cy,Cx), radius=radius)
    
    #compute descriptors in first window
    kps0_d, Ds0 = descriptor.compute_descriptors(win0, kps0)
    
    #loop over all windows (varying translation)
    d_mean = []
    d_std  = []
    d_min  = []
    d_max  = []
    for n in range(0,wins.shape[0]):
        Dang = Dangs[n]
        win  = wins[n]
        print Dang,
        
        #find keypoint locations in rotated window
        # from win0 to win1 is -Dang, center of window is Center of rotation
        kps1 = misc.rotate_kps( kps0, -Dang, centre=(Cy,Cx))
        
        #compute descriptors at same points in new window
        kps1_d, Ds1 = descriptor.compute_descriptors(win, kps1)

        #find distances between keypoint pairs
        d_correlated = distance_func(Ds0, Ds1)
        
        #find  statistics
        d_mean.append(d_correlated.mean())
        d_std.append(d_correlated.std())
        d_min.append(d_correlated.min())
        d_max.append(d_correlated.max())
        
    d_mean = np.array(d_mean)
    d_std = np.array(d_std)
    d_min = np.array(d_min)
    d_max = np.array(d_max)
    print 'done'
    
    
    #store results
    res = { 'd_uncorrelated':d_uncorrelated,
            'd_mean': d_mean,
            'd_std':d_std,
            'd_min':d_min,
            'd_max':d_max,
            'Dangs':Dangs, 
            'Ax':Ax,
            'Ay':Ay,
            'c':c, }

    misc.save_dict_to_hdf5(res , out, descriptor.method)
    
    #plot
    #plt.figure(filename)
    #l = plt.semilogy(Ax , d_mean, '.', label=descriptor.method)[0]
    #plt.hlines(d_uncorrelated.mean(), 0, 1100, colors=l.get_color())

#close output file
out_file.close()
#close input file
data.close()


