"""
Figure generation Descriptor translation descrimination

Uses distance from random speckle for the uncorrelated band
"""
#%%-----------------------------------------------------------------------------
# imports
#-------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt
plt.ion()

from mpl_toolkits.axes_grid1 import ImageGrid

#plotting helpers
import lib.plotting as plotting
import textwrap

import h5py

#surface preparation methods
surfaces = ( ('horizontal milling', (2.5,16,50)),
            ('flat lapping reaming', (0.55,1.6,10)),
            ('vertical milling', (2.5,16,50)),
            ('turning', (2.5,16,50)),
            ('grinding', (0.55,1.6,10))  )
            
#select surface to plot
surface, Rzs = surfaces[0]
Rz = Rzs[1]      

#%%-----------------------------------------------------------------------------
# 
#-------------------------------------------------------------------------------
def plot_bands(ax, x, d_mean, d_std, du, color='r', linecolor='k', linestyle='--', hatch='///'):
    """
    Plot correlated and uncorrelated bands at 1 sigma
    """
    du_mean  = du.mean()
    du_std   = du.std()
    i0 = 1 #cut off first point?

    #uncorrelated speckles
    ax.fill_between( np.arange(0,x[-1]), du_mean-du_std , du_mean+du_std, 
                        color=color, alpha=0.5, linewidth=1, hatch=hatch)
    
    ax.fill_between( np.arange(0,x[-1]), du_mean-(3*du_std) , du_mean+(3*du_std), 
                        color=color, alpha=0.25, linewidth=1, hatch=hatch)
                        
    #correlated speckles
    #ax.fill_between(x, d_mean-d_std , d_mean+d_std, 
    #                    color=color, alpha=0.5, linewidth=1,  hatch=hatch)
    ax.plot( x[i0:],  d_mean[i0:]-d_std[i0:], linestyle, color=linecolor)
    #ax.plot( x,  d_mean, '-', color=color)
    ax.plot( x[i0:],  d_mean[i0:]+d_std[i0:], linestyle, color=linecolor)
    
#%%-----------------------------------------------------------------------------
# paper figure translation
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)
cmap = plt.get_cmap('tab10')

#res_in = h5py.File('..\\results\\5.5.3 descrimination_translation.hdf5', 'r')
res_in = h5py.File('..\\results\\5.5.3 descrimination_translation v2.hdf5', 'r')

surface_name = '{0} Rz={1} um'.format(surface, Rz)
res = res_in[surface_name]
print 'Plotting for ', surface_name


# 2x4
f = plt.figure(figsize=(6.7,4.0))
ag = plotting.axis_grid( f, (-0.02,0.10,1.0,0.94), rowscols=(2,4), pad=(0.07,0.12))
filename = '..\\figures\\descriptor_discrimination_translation(2x4).pdf'
titles= ['USURF\n(64 value descriptor)', 'SURF\n(64 value descriptor)', 'ORB\n(16x16 pixel)', 'BRISK\n(16x16 pixel)',
         'USURF\n(128 value descriptor)', 'USURF\n(128 value descriptor)', 'ORB\n(32x32 pixel)', 'BRISK\n(32x32 pixel)']


#Axis 1 USURF
ax1 = ag[0]
ax1.set_title(titles[0], fontsize=8)
ax1.set_ylabel('Distance\n(L1norm)')
ax1.set_ylim(0,5)
ax1.yaxis.labelpad = -5
ax1.set_yticks( (0, 2, 4))
ax1.set_yticklabels( ('0','', '4'))
ax1.set_xlabel('Translation [px]')
ax1.set_xlim(0,1000)

g = res['USURF (length 64 descriptor)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax1, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#Axis 2 SURF
ax2 = ag[1]
ax2.set_title(titles[1], fontsize=8)
ax2.set_ylabel('Distance\n(L1norm)')
ax2.set_ylim(0,5)
ax2.yaxis.labelpad = -5
ax2.set_yticks( (0, 2, 4))
ax2.set_yticklabels( ('0','', '4'))
ax2.set_xlabel('Translation [px]')
ax2.set_xlim(0,1000)

g = res['SURF (length 64 descriptor)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax2, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax2.annotate('64 value', xy=(653, 2.0), xytext=(500, 0.4), arrowprops=dict(arrowstyle='->'),)
#ax2.annotate('128 value', xy=(198, 3.36), xytext=(50, 4.1), arrowprops=dict(arrowstyle='->'),)

#Axis 3 ORB
ax3 = ag[2]
ax3.set_title(titles[2], fontsize=8)
ax3.set_ylabel('Distance\n(Hamming)')
ax3.yaxis.labelpad = -15
ax3.set_yticks( (0, 64, 128))
ax3.set_yticklabels( ('0','','128'))
ax3.set_ylim(0,160)
ax3.set_xlabel('Translation [px]')
ax3.set_xlim(0,1000)

g = res['ORB (patchSize = 16)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax3, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax3.annotate('16x16\npixel', xy=(700, 94), xytext=(138, 82), arrowprops=dict(arrowstyle='->'),)
#ax3.annotate('32x32\npixel', xy=(542, 67), xytext=(680, 10), arrowprops=dict(arrowstyle='->'),)

#Axis 4 BRISK
ax4 = ag[3]
ax4.set_title(titles[3], fontsize=8)
ax4.set_ylabel('Distance\n(Hamming)')
ax4.yaxis.labelpad = -15
ax4.set_yticks( (0,128,256))
ax4.set_yticklabels(( '0','','256'))
ax4.set_ylim(0,320)
ax4.set_xlabel('Translation [px]')
ax4.set_xlim(0,1000)

g = res['BRISK (patchSize = 16)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax4, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')


#Axis 5 USURF 128
ax1 = ag[4]
ax1.set_title(titles[4], fontsize=8)
ax1.set_ylabel('Distance\n(L1norm)')
ax1.set_ylim(0,5)
ax1.yaxis.labelpad = -5
ax1.set_yticks( (0, 2, 4))
ax1.set_yticklabels( ('0','','4'))
ax1.set_xlabel('Translation [px]')
ax1.set_xlim(0,1000)

g = res['USURF (length 128 descriptor)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax1, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax1.annotate('64 value', xy=(700, 1.8), xytext=(500, 0.4), arrowprops=dict(arrowstyle='->'),)
#ax1.annotate('128 value', xy=(620, 3.3), xytext=(50, 4.0), arrowprops=dict(arrowstyle='->'),)

#Axis 6 SURF 128
ax2 = ag[5]
ax2.set_title(titles[5], fontsize=8)
ax2.set_ylabel('Distance\n(L1norm)')
ax2.set_ylim(0,5)
ax2.yaxis.labelpad = -5
ax2.set_yticks( (0, 2, 4))
ax2.set_yticklabels( ('0','','4'))
ax2.set_xlabel('Translation [px]')
ax2.set_xlim(0,1000)

g = res['SURF (length 128 descriptor)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax2, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax2.annotate('64 value', xy=(653, 2.0), xytext=(500, 0.4), arrowprops=dict(arrowstyle='->'),)
#ax2.annotate('128 value', xy=(198, 3.36), xytext=(50, 4.1), arrowprops=dict(arrowstyle='->'),)

#Axis 7 ORB 32x32
ax3 = ag[6]
ax3.set_title(titles[6], fontsize=8)
ax3.set_ylabel('Distance\n(Hamming)')
ax3.yaxis.labelpad = -15
ax3.set_yticks( (0, 64, 128))
ax3.set_yticklabels( ('0','','128'))
ax3.set_ylim(0,160)
ax3.set_xlabel('Translation [px]')
ax3.set_xlim(0,1000)

g = res['ORB (patchSize = 32)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax3, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax3.annotate('16x16\npixel', xy=(700, 94), xytext=(138, 82), arrowprops=dict(arrowstyle='->'),)
#ax3.annotate('32x32\npixel', xy=(542, 67), xytext=(680, 10), arrowprops=dict(arrowstyle='->'),)

#Axis 8 BRISK 32x32
ax4 = ag[7]
ax4.set_title(titles[7], fontsize=8)
ax4.set_ylabel('Distance\n(Hamming)')
ax4.yaxis.labelpad = -15
ax4.set_yticks( (0,128,256))
ax4.set_yticklabels(( '0','','256'))
ax4.set_ylim(0,320)
ax4.set_xlabel('Translation [px]')
ax4.set_xlim(0,1000)

g = res['BRISK (patchSize = 32)']
Ax = g['Ax']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax4, Ax, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax4.annotate('16x16\npixel', xy=(140, 180), xytext=(420, 155), arrowprops=dict(arrowstyle='->'),)
#ax4.annotate('32x32\npixel', xy=(340, 100), xytext=(680, 30), arrowprops=dict(arrowstyle='->'),)

#hide x axis inside
plotting.show_outside_xaxis_only(ag)

res_in.close()
f.savefig(filename, dpi=600)

#%%-----------------------------------------------------------------------------
# paper figure rotation
#-------------------------------------------------------------------------------
font = {'family' : 'sans',
        'weight' : 'normal',
        'size'   : 8}
plt.rc('font', **font)
cmap = plt.get_cmap('tab10')

#select surface to use
res_in = h5py.File('..\\results\\5.5.3 descrimination_rotation.hdf5', 'r')
surface_name = '{0} Rz={1} um'.format(surface, Rz)
res = res_in[surface_name]
print 'Plotting for ', surface_name

# 2x4
f = plt.figure(figsize=(6.7,4.0))
ag = plotting.axis_grid( f, (-0.02,0.10,1.0,0.94), rowscols=(2,4), pad=(0.07,0.12))
filename = '..\\figures\\descriptor_discrimination_rotation(2x4).pdf'
filename = '..\\figures\\figure15.pdf'

titles= ['USURF\n(64 value descriptor)', 'SURF\n(64 value descriptor)', 'ORB\n(16x16 pixel)', 'BRISK\n(16x16 pixel)',
         'USURF\n(128 value descriptor)', 'USURF\n(128 value descriptor)', 'ORB\n(32x32 pixel)', 'BRISK\n(32x32 pixel)']

#Axis 1 USURF
ax1 = ag[0]
ax1.set_title(titles[0], fontsize=8)
ax1.set_ylabel('Distance\n(L1norm)')
ax1.set_ylim(0,5)
ax1.yaxis.labelpad = -5
ax1.set_yticks( (0, 2, 4))
ax1.set_yticklabels( ('0','','4'))
ax1.set_xlabel('Rotation [$\degree$]')
#ax1.set_xlim(0,1000)

g = res['USURF (length 64 descriptor)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax1, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')


#Axis 2 SURF
ax2 = ag[1]
ax2.set_title(titles[1], fontsize=8)
ax2.set_ylabel('Distance\n(L1norm)')
ax2.set_ylim(0,5)
ax2.yaxis.labelpad = -5
ax2.set_yticks( (0, 2, 4))
ax2.set_yticklabels( ('0','','4'))
ax2.set_xlabel('Rotation [$\degree$]')
#ax2.set_xlim(0,1000)

g = res['SURF (length 64 descriptor)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax2, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax2.text( 100, 2.1, '64 value')
#ax2.text( 100, 3.9, '128 value')
#ax2.annotate('64 value', xy=(653, 2.0), xytext=(500, 0.4), arrowprops=dict(arrowstyle='->'),)
#ax2.annotate('128 value', xy=(198, 3.36), xytext=(50, 4.1), arrowprops=dict(arrowstyle='->'),)

#Axis 3 ORB
ax3 = ag[2]
ax3.set_title(titles[2], fontsize=8)
ax3.set_ylabel('Distance\n(Hamming)')
ax3.yaxis.labelpad = -15
ax3.set_yticks( (0, 64, 128))
ax3.set_yticklabels( ('0','','128'))
ax3.set_ylim(0,140)
ax3.set_xlabel('Rotation [$\degree$]')
#ax3.set_xlim(0,1000)

g = res['ORB (patchSize = 16)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax3, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax3.annotate('16x16\npixel', xy=(40, 100), xytext=(70, 40), arrowprops=dict(arrowstyle='->'),)
#ax3.annotate('32x32\npixel', xy=(246, 85), xytext=(180, 10), arrowprops=dict(arrowstyle='->'),)
#ax3.annotate('', xy=(300, 128), xytext=(235, 33), arrowprops=dict(arrowstyle='->'),)

#Axis 4 BRISK
ax4 = ag[3]
ax4.set_title(titles[3], fontsize=8)
ax4.set_ylabel('Distance\n(Hamming)')
ax4.yaxis.labelpad = -15
ax4.set_yticks( (0,128,256))
ax4.set_yticklabels(( '0','','256'))
ax4.set_ylim(0,300)
ax4.set_xlabel('Rotation [$\degree$]')
#ax4.set_xlim(0,1000)

g = res['BRISK (patchSize = 16)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax4, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax4.annotate('16x16\npixel', xy=(166, 200), xytext=(480, 155), arrowprops=dict(arrowstyle='->'),)
#ax4.annotate('32x32\npixel', xy=(700, 128), xytext=(680, 30), arrowprops=dict(arrowstyle='->'),)


#Axis 5 USURF 128
ax1 = ag[4]
ax1.set_title(titles[4], fontsize=8)
ax1.set_ylabel('Distance\n(L1norm)')
ax1.set_ylim(0,5)
ax1.yaxis.labelpad = -5
ax1.set_yticks( (0, 2, 4))
ax1.set_yticklabels( ('0','','4'))
ax1.set_xlabel('Rotation [$\degree$]')
#ax1.set_xlim(0,1000)

g = res['USURF (length 128 descriptor)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax1, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')


#Axis 6 SURF 128
ax2 = ag[5]
ax2.set_title(titles[5], fontsize=8)
ax2.set_ylabel('Distance\n(L1norm)')
ax2.set_ylim(0,5)
ax2.yaxis.labelpad = -5
ax2.set_yticks( (0, 2, 4))
ax2.set_yticklabels( ('0','','4'))
ax2.set_xlabel('Rotation [$\degree$]')
#ax2.set_xlim(0,1000)

g = res['SURF (length 128 descriptor)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax2, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')


#Axis 7 ORB 32x32
ax3 = ag[6]
ax3.set_title(titles[6], fontsize=8)
ax3.set_ylabel('Distance\n(Hamming)')
ax3.yaxis.labelpad = -15
ax3.set_yticks( (0, 64, 128))
ax3.set_yticklabels( ('0','','128'))
ax3.set_ylim(0,140)
ax3.set_xlabel('Rotation [$\degree$]')
#ax3.set_xlim(0,1000)

g = res['ORB (patchSize = 32)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax3, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#Axis 8 BRISK
ax4 = ag[7]
ax4.set_title(titles[7], fontsize=8)
ax4.set_ylabel('Distance\n(Hamming)')
ax4.yaxis.labelpad = -15
ax4.set_yticks( (0,128,256))
ax4.set_yticklabels(( '0','','256'))
ax4.set_ylim(0,300)
ax4.set_xlabel('Rotation [$\degree$]')
#ax4.set_xlim(0,1000)

g = res['BRISK (patchSize = 32)']
Dangs = g['Dangs']
c  = g['c']
d_mean  = g['d_mean'][:]
d_min   = g['d_min'][:]
d_max   = g['d_max'][:]
d_std  = g['d_std'][:]
d_uncorrelated = g['d_uncorrelated'][:]
plot_bands(ax4, Dangs, d_mean, d_std, d_uncorrelated, linestyle='-', color=cmap(0), hatch='')

#ax4.annotate('16x16\npixel', xy=(166, 200), xytext=(480, 155), arrowprops=dict(arrowstyle='->'),)
#ax4.annotate('32x32\npixel', xy=(700, 128), xytext=(680, 30), arrowprops=dict(arrowstyle='->'),)


plotting.set_xlimits(ag, 0, 360)
plotting.set_xticks(ag, (0,90,180,270,360))
#hide x axis inside
plotting.show_outside_xaxis_only(ag)

res_in.close()
f.savefig(filename, dpi=600)
