"""
Plotting helpers for multiple axis
"""
def axis_grid( fig, rect=(0.1,0.1,0.8,0.8), rowscols=(2,2), pad=(0.2,0.2)):
    """
    Create a grid of axis objects
    
    Returns a list of axis from top-left.
    Sets the ax.y_ouside and ax.x_outside flags to allow easy turn off of axis labels/ticks
    """ 
    rows,cols = rowscols
    axs = []
    w = (rect[2] - (cols*pad[0]))/cols
    h = (rect[3] - (rows*pad[1]))/rows
    for i in range(0,rows):
        for j in range(0, cols):
            x0 = rect[0] + pad[0]  +  j*(pad[0]+w)
            y0 = rect[1] + (rows-1-i)*(pad[1]+h)
            ax = fig.add_axes( (x0,y0,w,h))
            axs.append(ax)
            
            #mark outside axis flags
            ax.x_outside = False
            ax.y_outside = False
            
            if i==rows-1: ax.x_outside = True
            if j==0     : ax.y_outside = True

    return axs
 
def show_outside_axis_only(axs, labels = False, ticklabels=False, ticks=True):
    """
    Turn off axis labels, ticklabels and ticks.
    Uses the x_outside / y_outside flags set by axis_grid
    """
    for ax in axs:
        if ax.x_outside is False:
            if labels is False:  ax.set_xlabel('')
            if ticklabels is False:  ax.set_xticklabels([])
            if ticks is False: ax.set_xticks([])
        if ax.y_outside is False:
            if labels is False:  ax.set_ylabel('')
            if ticklabels is False:  ax.set_yticklabels([])
            if ticks is False: ax.set_yticks([])
            
def show_outside_xaxis_only(axs, labels = False, ticklabels=False, ticks=True):
    """
    Turn off axis labels, ticklabels and ticks.
    Uses the x_outside set by axis_grid
    """
    for ax in axs:
        if ax.x_outside is False:
            if labels is False:  ax.set_xlabel('')
            if ticklabels is False:  ax.set_xticklabels([])
            if ticks is False: ax.set_xticks([])


def set_limits( axs, xlimits, ylimits):
    """
    Set the x and y limits on all axis in list axs
    """
    for ax in axs:
        ax.set_xlim( *xlimits)
        ax.set_ylim( *ylimits)
        
def set_xlimits( axs, xmin, xmax):
    """
    Set the x limits on all axis in list axs
    """
    for ax in axs:
        ax.set_xlim( xmin, xmax) 

def set_ylimits( axs, ymin, ymax):
    """
    Set the y limits on all axis in list axs
    """
    for ax in axs:
        ax.set_ylim( ymin, ymax) 
        
def set_ticks( axs, xticks, yticks):
    """
    Set the x and y ticks on all axis in list axs
    """
    for ax in axs:
        ax.set_xticks( xticks)
        ax.set_yticks( yticks)

def set_xticks( axs, xticks):
    """
    Set the x ticks on all axis in list axs
    """
    for ax in axs:
        ax.set_xticks( xticks)
    